// Ionic Starter App
angular.module('starter', ['ionic', 'pascalprecht.translate', 'ionic.contrib.drawer', 'ngTwitter', 'starter.controllers', 'starter.services', 'ionic-datepicker', 'ngMessages', 'ngCordova', 'ngCordovaBeacon', 'ngSanitize', 'ngCordovaOauth', 'ngMask'])

        .run(function ($ionicPlatform, $translate, $http, Globals) {

            // Getting company data from json file
            $http.get('company_data.json').success(function (data) {
                console.log("Company Data Loaded");
                window.localStorage.setItem("COMPANY_INFO", angular.toJson(data));
            }).error(function (error) {
                console.log("Error on loading company data: " + error);
            });

            $ionicPlatform.ready(function () {

                Globals.START_TIMESTAMP = new Date().getTime() / 1000;

                // Cheking device settings to set app language
                if (typeof navigator.globalization !== "undefined") {
                    navigator.globalization.getPreferredLanguage(function (language) {
                        $translate.use((language.value).split("-")[0]).then(function (data) {
                            console.log("SUCCESS LANG DETECT -> " + data);
                        }, function (error) {
                            console.log("ERROR LANG DETECT -> " + error);
                            $translate.use("en");
                        });

                    }, null);
                } else {
                    console.log("Language undefined");
                }
                //$translate.use("es");

                // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
                // for form inputs)
                if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
                    cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
                    cordova.plugins.Keyboard.disableScroll(true);
                }
                if (window.StatusBar) {
                    StatusBar.styleDefault();
                }
                // No se tiene en cuenta el tamaño del texto de las preferencias del dispositivo
                if (window.MobileAccessibility) {
                    window.MobileAccessibility.usePreferredTextZoom(false);
                }
            });
        })
        // Configuring translate provider
        .config(function ($translateProvider) {
            $translateProvider.useStaticFilesLoader({
                'prefix': 'lang/',
                'suffix': '.json'
            });
            $translateProvider.preferredLanguage("en");
            $translateProvider.fallbackLanguage("en");
            $translateProvider.forceAsyncReload(true);
            //WARNING: It is good to have this property ON to prevent injections
            //$translateProvider.useSanitizeValueStrategy('escape');
        })
        .config(function ($stateProvider, $urlRouterProvider) {

            $stateProvider

                    .state('welcome', {
                        cache: false,
                        url: '/welcome',
                        templateUrl: 'templates/welcome.html',
                        controller: 'WelcomeController'
                    })

                    .state('register', {
                        cache: false,
                        url: '/register',
                        templateUrl: 'templates/register.html',
                        controller: 'RegisterController'
                    })

                    .state('app', {
                        url: '/app',
                        cache: true,
                        abstract: true,
                        templateUrl: 'templates/sideMenu.html',
                        controller: 'AppController'
                    })

                    .state('app.searchContents', {
                        cache: false,
                        url: '/searchContents',
                        views: {
                            'menuContent': {
                                templateUrl: 'templates/searchContents.html',
                                controller: 'SearchContentsController'
                            }
                        }
                    })

                    .state('app.contentView', {
                        cache: false,
                        url: '/contentView',
                        views: {
                            'menuContent': {
                                templateUrl: 'templates/contentView.html',
                                controller: 'ContentViewController'
                            }
                        }
                    })

                    .state('app.contact', {
                        cache: false,
                        url: '/contact',
                        views: {
                            'menuContent': {
                                templateUrl: 'templates/contact.html',
                                controller: 'ContactController'
                            }
                        }
                    })

                    .state('app.favorites', {
                        cache: false,
                        url: '/favorites',
                        views: {
                            'menuContent': {
                                templateUrl: 'templates/favorites.html',
                                controller: 'FavoritesController'
                            }
                        }
                    })

                    .state('app.userProfile', {
                        cache: false,
                        url: '/userProfile',
                        views: {
                            'menuContent': {
                                templateUrl: 'templates/userProfile.html',
                                controller: 'UserProfileController'
                            }
                        }
                    })

                    .state('app.settings', {
                        cache: false,
                        url: '/settings',
                        views: {
                            'menuContent': {
                                templateUrl: 'templates/settings.html',
                                controller: 'SettingsController'
                            }
                        }
                    });

            // Se comprueba si hay un API-KEY introducido
            if (window.localStorage.getItem("api_key") === null || window.localStorage.getItem("api_key") === "") {
                // Si no hay se manda a la pantalla de bienvenida
                $urlRouterProvider.otherwise('/welcome');

            } else if ((window.localStorage.getItem("app_id") === null || window.localStorage.getItem("app_id") === "")
                    && window.localStorage.getItem("api_key") !== "Mb7I-cH0h-RrEU-Yl3m") {
                // Si no hay ningun usuario registrado se lanza el registro y si no es demo
                $urlRouterProvider.otherwise('/register');

            } else {
                // Si un usuario está registrado se lanza el panel principal
                $urlRouterProvider.otherwise('/app/searchContents');
            }
        });