/* global angular */
angular.module('starter.controllers').controller('AppController', ['$scope', '$translate', '$state', '$ionicPopup', '$ionicModal', '$ionicPlatform', 'Globals', 'PreferencesService', 'PlatformService', '$rootScope', 'PushNotificationsService',
    function ($scope, $translate, $state, $ionicPopup, $ionicModal, $ionicPlatform, Globals, PreferencesService, PlatformService, $rootScope, PushNotificationsService) {

        $scope.content = {};

        $rootScope.myService = "";

        $ionicPlatform.ready(function () {
            console.log("READY APP CONTROLLER");
            $scope.content = Globals.INFO().CONTACT_DATA;

            var backgroundMode = PreferencesService.getBackgroundMode();
            // Enable/Disable background mode
            if (backgroundMode) {
                cordova.plugins.backgroundMode.enable();
            } else {
                cordova.plugins.backgroundMode.disable();
            }

            $rootScope.myService = cordova.plugins.easyContextService;
            $rootScope.myService.getStatus(function (serviceStatus) {
                console.log("BService: " + JSON.stringify(serviceStatus));
                // If the background service was lunched after boot, we should stop it when the app is opened
                if (serviceStatus.ServiceRunning) {
                    var config = {
                        "ENABLED": false
                    };
                    $rootScope.myService.setConfiguration(config, function (setconfig) {
                        console.log("Set Config Response: " + JSON.stringify(setconfig));

                        // Stopping background service
                        $rootScope.myService.stopService(function (r) {
                            console.log("Stop MyService: " + JSON.stringify(r));
                        }, function (e) {
                            console.log("Stop MyService Error: " + JSON.stringify(e));
                        });
                    }, function (errorconfig) {
                        console.log("Set Config Error: " + JSON.stringify(errorconfig));
                    });
                }

            }, function (e) {
                console.log("MyService Error: " + JSON.stringify(e));
            });

            // Android customization
            cordova.plugins.backgroundMode.setDefaults({
                text: 'Background Service is Running',
                title: "EasyContextDemo App Module 2",
                // This will show/hide notification about running app in background
                silent: false
            });

            // Called when background mode has been activated: on pause
            cordova.plugins.backgroundMode.onactivate = function () {
                $rootScope.myService = cordova.plugins.easyContextService;
                // Starting service 
                $rootScope.myService.startService(function (r) {
                    console.log("Start MyService: " + JSON.stringify(r));

                    // Configuring the time for waiting to start beacon search
                    timerWait = Globals.BACKGROUND_BEACON_SEARCH_DURATION;
                    if (PreferencesService.getBackgroundBatterySaverMode()) {
                        // In battery saver mode is about 5 + 30 = 35 seconds
                        timerWait += Globals.BACKGROUND_BEACON_SEARCH_WAIT_BATTERY_SAVER;
                    } else {
                        // By default is about 5 + 5 = 10 seconds
                        timerWait += Globals.BACKGROUND_BEACON_SEARCH_WAIT_DEFAULT;
                    }
                    // Setting service configurations
                    var config = {
                        "BeaconUUID": PreferencesService.getUuid(),
                        "ENABLED": true,
                        "APP_ID": PreferencesService.getAppId(),
                        "API_KEY": PreferencesService.getApiKey(),
                        "REG_TOKEN": PreferencesService.getPushToken(),
                        "TIMER_MILLISECONDS": (timerWait * 1000),
                        "USE_GEO": PreferencesService.getGeolocation(),
                        "SEARCH_DURATION": (Globals.BACKGROUND_BEACON_SEARCH_DURATION * 1000)
                    };

                    console.log(JSON.stringify(config));
                    $rootScope.myService.setConfiguration(config, function (setconfig) {
                        console.log("Set Config Response: " + JSON.stringify(setconfig));
                    }, function (errorconfig) {
                        console.log("Set Config Error: " + JSON.stringify(errorconfig));
                    });

                    // Setting timer of the service for 1 minute
                    $rootScope.myService.enableTimer((timerWait * 1000), function (timer_response) {
                        console.log("Enable Timer MyService: " + JSON.stringify(timer_response));
                    }, function (timer_error) {
                        console.log("Enable Timer Error MyService: " + JSON.stringify(timer_error));
                    });

                }, function (e) {
                    console.log("Start MyService Error: " + JSON.stringify(e));
                });
            };

            // Called when background mode has been deactivated: on resume
            cordova.plugins.backgroundMode.ondeactivate = function () {
                // destroy here some task if they was started running onactivate event
                console.log("Back from background");
                $rootScope.myService = cordova.plugins.easyContextService;

                var config = {
                    "ENABLED": false
                };
                $rootScope.myService.setConfiguration(config, function (setconfig) {
                    console.log("Set Config Response: " + JSON.stringify(setconfig));
                    $rootScope.myService.stopService(function (r) {
                        console.log("Stop MyService: " + JSON.stringify(r));
                    }, function (e) {
                        console.log("Stop MyService Error: " + JSON.stringify(e));
                    });
                }, function (errorconfig) {
                    console.log("Set Config Error: " + JSON.stringify(errorconfig));
                });
            };

            // Called when background mode has been failed
            cordova.plugins.backgroundMode.onfailure = function (errorCode) {
                console.log("BGM Error: " + errorCode);
            };

        });

        // Evento al pulsar el boton de logout
        $scope.logout = function () {

            $translate(['BUTTON_NO', 'BUTTON_YES', 'APP_LOGOUT_TEXT', 'APP_LOGOUT_TITLE']).then(function (translations) {
                $ionicPopup.confirm({
                    title: translations.APP_LOGOUT_TITLE,
                    template: translations.APP_LOGOUT_TEXT,
                    buttons: [
                        {
                            text: translations.BUTTON_NO,
                            type: 'button-default',
                            onTap: function (e) {}
                        },
                        {
                            text: translations.BUTTON_YES,
                            type: 'button-assertive',
                            onTap: function (e) {
                                PushNotificationsService.unregisterPushNotifications();
                                cordova.plugins.backgroundMode.disable();
                                PreferencesService.logout();
                                Globals.LAST_DISCOVERED_CONTENTS = "";
                                Globals.START_SEARCHING_CONTENTS = false;
                                Globals.SELECTED_CONTENT = {};
                                $state.go('welcome');
                            }
                        }
                    ]
                });
            });

        };

        $scope.getAccountAccess = function () {
            return PreferencesService.getAccountAccess() == "true" && PreferencesService.getAccountMode() != 0;
        };
    }]);