/* global angular */
angular.module('starter.controllers').controller('ContentViewController', ['$scope', '$translate', '$state', '$ionicPlatform', '$cordovaSocialSharing', 'Globals', 'PreferencesService', 'PlatformService', '$sce', 'Globals', 'PushNotificationsService',
    function ($scope, $translate, $state, $ionicPlatform, $cordovaSocialSharing, Globals, PreferencesService, PlatformService, $sce, Globals, PushNotificationsService) {

        $scope.content = {};
        $scope.isLandscape = false;
        $scope.isInFavorites = false;
        
        $ionicPlatform.ready(function () {
            $scope.content = Globals.SELECTED_CONTENT;
        });

        $scope.trustSrc = function (src) {
            return $sce.trustAsResourceUrl(src);
        }

        // Evento al entrar a la vista
        $scope.$on("$ionicView.enter", function (event, data) {
            $scope.isInFavorites = PreferencesService.favoriteContentExists($scope.content.id);
            console.log("In Favorites: " + $scope.isInFavorites);
            checkOrientation();
            screen.unlockOrientation();

             // Register push notifications if not registered yet
            PushNotificationsService.registerPushNotifications();
        });
        
        // Evento al pulsar el boton de volver atras
        $scope.back = function () {
            var backRoute = "app.searchContents";
            if (Globals.CONTENTVIEW_OPENED_FROM === "favorites") {
                backRoute = "app.favorites";
            }
            $state.go(backRoute);
        };
        // Event to save/remove the content from favorite list
        $scope.favorite = function () {
            $translate(['CONTENT_FAV_ADDED', 'CONTENT_FAV_REMOVED']).then(function (translations) {
                if (!PreferencesService.favoriteContentExists($scope.content.id)) {
                    PreferencesService.addFavoriteContent($scope.content);
                    $scope.isInFavorites = true;
                } else {
                    PreferencesService.removeFavoriteContent($scope.content.id);
                    $scope.isInFavorites = false;
                }
            });
        };
        // Evento al pulsar el boton de compartir
        $scope.shareContent = function () {

            // Se obtiene el token del contenido
            var apiKey = PreferencesService.getApiKey() == null ? Globals.API_KEY : PreferencesService.getApiKey();
            var token = md5($scope.content.id + apiKey);
            // Se crea el enlace para compartir
            var url = Globals.URL_SHARE_CONTENT;
            url = url.replace("<id_content>", $scope.content.id);
            url = url.replace("<token>", token);
            console.log("URL: " + url);
            // Se comparte el contenido
            $cordovaSocialSharing.share("Hello! this could be of your interest!\n", "This could be of your interest!", "", $scope.content.title + " : " + url).then(function (result) {
                console.log("SUCCESS: " + result);
                // Se registra la actividad del usuario
                PlatformService.logUserInteraction(PreferencesService.getAppId(), apiKey, "social", $scope.content.id, "").success(function (data) {
                    console.log("SUCCESS: " + data);
                }).error(function (error) {
                    console.log("ERROR: " + error);
                });
            }, function (err) {
                console.log("ERROR: " + err);
            });
        };
        // Función para comprobar la orientación de la pantalla
        function checkOrientation() {
            switch (window.orientation) {
                case 0:
                case 180:
                    //portrait
                    $scope.isLandscape = false;
                    break;
                case 90:
                case -90:
                    //landscape
                    $scope.isLandscape = true;
                    break;
            }
        }

        // Evento al cambiar la orientación de la pantalla
        window.addEventListener("orientationchange", function () {
            checkOrientation();
            console.log("Landscape: " + $scope.isLandscape);
            //this method is to update the bindings of the content view
            $scope.$apply();
        });
        

    }]);