/* global angular */
angular.module('starter.controllers').controller('RegisterController', ['$scope', '$translate', '$twitterApi', '$state', '$ionicLoading', '$ionicPlatform', '$cordovaDevice', '$timeout', 'Globals', 'PreferencesService', 'PlatformService', 'ionicDatePicker', '$cordovaOauth', 'SocialProviders', '$sce', 'Utils',
    function ($scope, $translate, $twitterApi, $state, $ionicLoading, $ionicPlatform, $cordovaDevice, $timeout, Globals, PreferencesService, PlatformService, ionicDatePicker, $cordovaOauth, SocialProviders, $sce, Utils) {

        $scope.signUpData = {};
        $scope.accountMode = 0;
        var apiKey = "";
        var appID = "";

        $ionicPlatform.ready(function () {
            console.log("READY REGISTER");

            // Gets account mode and API-KEY
            $scope.accountMode = PreferencesService.getAccountMode();
            apiKey = PreferencesService.getApiKey() == null ? Globals.API_KEY : PreferencesService.getApiKey();

            // Generate app id
            appID = "da";
            appID += (cordova.platformId == 'ios') ? "i" : "a";
            appID += $cordovaDevice.getUUID();
            console.log("APP ID: " + appID);

            // Se previene el volver atras
            $ionicPlatform.registerBackButtonAction(function (event) {
                PreferencesService.setApiKey("");
                $state.go('welcome');
            }, 100);
        });

        $scope.$on("$ionicView.beforeEnter", function (event, data) {
            $scope.signUpData.gender = "female";
        });

        // Evento al pulsar sobre el selector de fecha de nacimiento
        $scope.onBirthdateClick = function () {
            var inpDate;
            if ($scope.signUpData.birthdate !== undefined) {
                inpDate = new Date($scope.signUpData.birthdate);
            } else {
                inpDate = new Date();
            }
            var birthdateInputConfig = {
                callback: function (val) {  //Mandatory
                    $scope.signUpData.birthdate = Utils.formatDate(new Date(val));
                },
                from: new Date(1900, 1, 1), //Optional
                to: new Date(), //Optional
                inputDate: inpDate, //Optional
                mondayFirst: true, //Optional
                disableWeekdays: [], //Optional
                closeOnSelect: false, //Optional
                templateType: 'popup'       //Optional
            };
            ionicDatePicker.openDatePicker(birthdateInputConfig);
        };

        // Funcion para registrarse por defecto
        $scope.signUp = function () {
            updateUserData($scope.signUpData.username, $scope.signUpData.email, $scope.signUpData.birthdate, $scope.signUpData.gender, "", "");
        };
        // Facebook login
        $scope.facebookLogin = function () {
            // Se lanza el formulario de inicio de sesión en Facebook
            $cordovaOauth.facebook(Globals.INFO().FACEBOOK_CLIENT_ID, ["email", "public_profile", "user_birthday"], {redirect_uri: "http://localhost/callback"}).then(function (result) {
                // Se obtiene la información del usuario
                SocialProviders.getFacebookProfileInfo(result.access_token).success(function (userdata) {
                    console.log(JSON.stringify(userdata));
                    updateUserData(userdata.data.name, userdata.data.email, userdata.data.birthday, userdata.data.gender, "facebook", result.access_token);
                }).error(function (data) {
                    console.log("Error: " + JSON.stringify(data));
                });
            }, function (error) {
                console.log("Error: " + JSON.stringify(error));
            });
        };
        
        // Twitter login
        $scope.twitterLogin = function () {
            $cordovaOauth.twitter(Globals.INFO().TWITTER_CONSUMER_KEY, Globals.INFO().TWITTER_CONSUMER_SECRET).then(function (result) {
                console.log(JSON.stringify(result));
                // Configuring the twitter api with recieved tokens (Required by API)
                $twitterApi.configure(Globals.INFO().TWITTER_CONSUMER_KEY, Globals.INFO().TWITTER_CONSUMER_SECRET, result);
                var USER_INFO_URL = 'https://api.twitter.com/1.1/account/verify_credentials.json';
                // Making request to get more detailed information about user, including email address
                $twitterApi.getRequest(USER_INFO_URL, {include_email: true, include_entities: false}).then(function (data) {
                    console.log(JSON.stringify(data));
                    updateUserData(data.name, data.email, data.birthday, data.gender, "twitter", result.oauth_token);
                }, function (error) {
                    console.log('Error [Twitter Info]: ' + error);
                });
            }, function (error) {
                console.log("Error [Twitter Login]: " + JSON.stringify(error));
            });
        };

        function updateUserData(fullName, email, birthday, gender, loginProvider, accessToken) {

            if ($scope.accountMode == 1 || $scope.accountMode == null) {
                // Si es una cuenta de usuario o es una aplicación en producción
                registerUser("", birthday, gender, loginProvider, accessToken);
            } else {
                // Si es un registro para crear una cuenta nueva en la plataforma
                registerClient(fullName, email, birthday, gender, loginProvider, accessToken)
            }
        }

        // Función para registrar un usuario de la aplicación
        function registerUser(fullName, birthday, gender, loginProvider, accessToken) {
            console.log("REGISTER USER");
            $ionicLoading.show({
                template: '<ion-spinner icon="android"></ion-spinner>'
            });
            PlatformService.updateUserData(appID, apiKey, birthday, gender, fullName, "").success(function (data) {
                console.log("Success: " + JSON.stringify(data));
                // Se almacenan los datos de inicio de sesión del usuario
                PreferencesService.setLoginData(appID, fullName, birthday, gender, loginProvider, accessToken);
                // Se oculta el spinner de cargando
                $ionicLoading.hide();
                PreferencesService.setPushNotificationsEnable(true);
                // Accedemos al menu principal
                $state.go('app.searchContents');
            }).error(function (data) {
                console.log("Error: " + data);
                // Se oculta el spinner de cargando
                $ionicLoading.hide();
                //TODO: If the API-KEY is Pro it will return, Unauthorized2. Maybe show the message
            });
        }

        // Función para registrar un cliente de la plataforma
        function registerClient(fullName, email, birthday, gender, loginProvider, accessToken) {
            console.log("REGISTER CLIENT");
            $ionicLoading.show({
                template: '<ion-spinner icon="android"></ion-spinner>'
            });
            $translate([
                'REGISTER_EMAIL_USED'
            ]).then(function (translations) {
                // Se registra el cliente en la plataforma
                PlatformService.registerClient(fullName, email).success(function (data) {
                    console.log("Success: " + data);
                    apiKey = data;
                    // Se guarda el API-KEY
                    PreferencesService.setApiKey(apiKey);
                    // Se registra tambien como usuario de la aplicación
                    PlatformService.updateUserData(appID, apiKey, birthday, gender, fullName, "").success(function (data) {
                        console.log("Success: " + data);
                        // Se almacenan los datos de inicio de sesión del usuario
                        PreferencesService.setLoginData(appID, fullName, birthday, gender, loginProvider, accessToken);
                        // Se añade por defecto el UUID
                        PreferencesService.setUuid(Globals.UUID_DEMO_ACCOUNT);
                        // Se oculta el spinner de cargando
                        $ionicLoading.hide();
                        PreferencesService.setPushNotificationsEnable(true);
                        // Accedemos al menu principal
                        $state.go('app.searchContents');
                    }).error(function (data) {
                        console.log("Error: " + data);
                    });
                }).error(function (data) {
                    // Error 401 The email is not available
                    console.log("Error: " + data);
                    showToastMessage(translations.REGISTER_EMAIL_USED);
                });
            });
        }

        // Evento al pulsar el boton de volver atras
        $scope.back = function () {
            PreferencesService.setApiKey("");
            $state.go('welcome');
        };
    }]);