/* global angular */
angular.module('starter.controllers').controller('SearchContentsController', ['$scope', '$translate', '$state', '$ionicPopup', '$ionicPlatform', '$rootScope', '$timeout', '$ionicLoading', '$cordovaGeolocation', 'Globals', 'PreferencesService', 'IBeaconService', 'PlatformService', '$sce', 'Utils', 'PushNotificationsService',
    function ($scope, $translate, $state, $ionicPopup, $ionicPlatform, $rootScope, $timeout, $ionicLoading, $cordovaGeolocation, Globals, PreferencesService, IBeaconService, PlatformService, $sce, Utils, PushNotificationsService) {

        $scope.contents = [];

        $scope.companyName = Globals.INFO().CONTACT_DATA.title;

        var uuid = "";
        var apiKey = "";
        var simulateBeaconsEnable = false;
        var geolocationEnable = false;
        var autodiscovering = false;

        var latitude = "";
        var longitude = "";
        var alertGeolocationPermission = false;

        $scope.infoMessage = '';
        $scope.infoMessageStyle = '';

        var searchingTimer;
        var searching = false;
        var runningAutodiscoveringProgressbar = false;
        var noContentsDiscoveredCurrentTimes = 0;

        $scope.showRefreshButton = false;
        $scope.showSpinner = false;
        $scope.showProgressbar = false;

        // Event when users tap back button in android devices
        $ionicPlatform.registerBackButtonAction(function (event) {
            console.log("Back device button clicked");
            if ($state.current.name === "app.searchContents") {
                //navigator.app.exitApp();
                ionic.Platform.exitApp();
            } else if ($state.current.name === "contentView") {
                var backRoute = "app.searchContents";
                if (Globals.CONTENTVIEW_OPENED_FROM === "favorites") {
                    backRoute = "app.favorites";
                }
                $state.go(backRoute);
            } else {
                $state.go('app.searchContents');
            }
        }, 100);

        // Event before enter to the current view
        $scope.$on("$ionicView.beforeEnter", function (event, data) {

            // Check if contents have been discovered before
            if (Globals.LAST_DISCOVERED_CONTENTS != "") {
                $scope.contents = JSON.parse(Globals.LAST_DISCOVERED_CONTENTS);
            }
        });

        $ionicPlatform.ready(function () {
            console.log("READY SEARCH CONTENTS");

            // Locks screen in portrait mode
            screen.lockOrientation('portrait');
            $scope.showProgressbar = false;

            // Gets mobile OS
            Globals.PLATFORM = cordova.platformId;
            console.log("Mobile OS: " + Globals.PLATFORM);

            // Gets UUID, API-KEY and if app is in beacon simulation mode or not
            uuid = PreferencesService.getUuid() == null ? Globals.UUID : PreferencesService.getUuid();
            apiKey = PreferencesService.getApiKey() == null ? Globals.API_KEY : PreferencesService.getApiKey();
            simulateBeaconsEnable = PreferencesService.getBeaconsSimulation();
            geolocationEnable = PreferencesService.getGeolocation();
            autodiscovering = PreferencesService.getAutoDiscovering();
            autodiscoveringTimer = PreferencesService.getAutoDiscoveringTimer() == null ? 15 : PreferencesService.getAutoDiscoveringTimer();
            alertGeolocationPermission = true;
            console.log("UUID: " + uuid + ", API-KEY: " + apiKey + ", SIMULATE BEACONS: " + simulateBeaconsEnable + ", GEOLOCATION: " + geolocationEnable + ", AUTO DISCOVERING: " + autodiscovering + ", AUTO DISCOVERING TIMER: " + autodiscoveringTimer);

            // Register push notifications if not registered yet
            PushNotificationsService.registerPushNotifications();

            // Check if the push receieved and needs to start searching automatically
            if (Globals.START_SEARCHING_CONTENTS) {
                console.log("ONREADY Search contents opened for push");
                changeMessage("SEARCHING_TXT");
                showSpinner();
                // Check if mobile has ble enable
                checkBle();
            } else if (autodiscovering) { // Check if autodiscovering is enable
                console.log("Auto discovering enable");
                if (Globals.LAST_DISCOVERED_CONTENTS != "") {
                    showRefreshButton();
                } else {
                    showSpinner();
                    // Check if mobile has ble enable
                    checkBle();
                }
            } else {
                if (Globals.LAST_DISCOVERED_CONTENTS == "") {
                    changeMessage("SEARCH_TAP_REFRESH_TXT");
                }
                showRefreshButton();
            }

        });

        // Event when user tap refresh button
        $scope.refresh = function () {
            if (!searching) {
                if (!autodiscovering) {
                    changeMessage("SEARCHING_TXT");
                }
                showSpinner();
                // Check if mobile has ble enable
                checkBle();
            }
        };

        function checkBle() {
            if (!searching) {
                searching = true;

                // If simulate beacons is enable, app does not require ble
                if (simulateBeaconsEnable) {
                    checkInternet();

                } else {
                    ble.isEnabled(function () {
                        console.log("1 - BLE is enable");
                        // Check if mobile has internet connection
                        checkInternet();

                    }, function () {
                        if (Globals.PLATFORM == 'ios') {
                            $translate(['MSG_BLE_DIS_TITLE', 'MSG_BLE_DIS_ALERT']).then(function (translations) {
                                $ionicPopup.alert({
                                    title: translations.MSG_BLE_DIS_TITLE,
                                    template: translations.MSG_BLE_DIS_ALERT
                                }).then(function (res) {
                                    interruptSearching();
                                });
                            });

                        } else if (Globals.PLATFORM == 'android') {
                            ble.enable(function () {
                                searching = true;
                                checkInternet();
                            },
                                    function () {
                                        $translate(['MSG_BLE_DIS_TITLE', 'MSG_BLE_DIS_ALERT']).then(function (translations) {
                                            $ionicPopup.alert({
                                                title: translations.MSG_BLE_DIS_TITLE,
                                                template: translations.MSG_BLE_DIS_ALERT
                                            }).then(function (res) {
                                                interruptSearching();
                                            });
                                        });
                                    });
                        }
                    });
                }
            }
        }

        function checkInternet() {
            if (window.Connection && navigator.connection.type != Connection.NONE) {
                console.log("2 - Internet is available");
                // Check location permission
                checkLocation();
            } else {
                console.log('Network: Cannot find Window.Connection');
                // Show no internet connection warning
                $translate(['MSG_NO_INTERNET_TITLE', 'MSG_NO_INTERNET_TEMPLATE']).then(function (translations) {
                    $ionicPopup.alert({
                        title: translations.MSG_NO_INTERNET_TITLE,
                        template: translations.MSG_NO_INTERNET_TEMPLATE
                    }).then(function (res) {
                        interruptSearching();
                    });
                });
            }
        }

        function interruptSearching() {
            searching = false;
            showRefreshButton();
            changeMessage("");
        }

        function checkLocation() {
            if (Globals.PLATFORM == 'android' && !simulateBeaconsEnable) {

                // Getting user mobile OS version
                var currentPlatformVersion = ionic.Platform.version();
                console.log("3 - Mobile OS version: " + currentPlatformVersion);

                // If Android device version is greater or equal than 6 it needs to request for location authorization
                if (currentPlatformVersion >= 6) {
                    cordova.plugins.diagnostic.getLocationAuthorizationStatus(function (status) {
                        console.log("LocationAuthorizationStatus: " + status);
                        if (status == "GRANTED") {
                            checkGeolocation();
                        } else {
                            cordova.plugins.diagnostic.requestLocationAuthorization(function (status) {
                                if (status == "GRANTED") {
                                    checkGeolocation();
                                } else {
                                    $translate(['MSG_LOC_PERM_DENY_TITLE', 'MSG_LOC_PERM_DENY_ALERT_ANDROID']).then(function (translations) {
                                        $ionicPopup.alert({
                                            title: translations.MSG_LOC_PERM_DENY_TITLE,
                                            template: translations.MSG_LOC_PERM_DENY_ALERT_ANDROID
                                        }).then(function (res) {
                                            interruptSearching();
                                        });
                                    });
                                }
                            }, function (error) {
                                console.log(error);
                                interruptSearching();
                            });

                        }
                    }, function (error) {
                        console.log(error);
                        interruptSearching();
                    });
                } else {
                    checkGeolocation();
                }

            } else if (Globals.PLATFORM == 'ios' && !simulateBeaconsEnable) {
                //iOS with cordova beacon plugin is not able to search beacons if locations is not avilable 
                $cordovaGeolocation.getCurrentPosition({timeout: 5000, enableHighAccuracy: true}).then(function (position) {
                    var latitude = position.coords.latitude;
                    var longitude = position.coords.longitude;
                    console.log("iOS: Latitude: " + latitude + " Longitude: " + longitude);
                    checkGeolocation();
                }, function (err) {
                    console.log("ios error on getCurrentPosition: " + JSON.stringify(err));

                    if (err.code == 1 && err.message == '') {
                        // Location permission denied
                        $translate(['MSG_LOC_PERM_DENY_TITLE', 'MSG_LOC_PERM_DENY_ALERT_IOS_BEACON']).then(function (translations) {
                            $ionicPopup.alert({
                                title: translations.MSG_LOC_PERM_DENY_TITLE,
                                template: translations.MSG_LOC_PERM_DENY_ALERT_IOS_BEACON + Globals.INFO().APPLICATION_NAME + "</b>"
                            }).then(function (res) {
                                interruptSearching();
                            });
                        });
                    } else {
                        // Location services are disabled
                        $translate(['MSG_LOC_DIS_TITLE', 'MSG_LOC_DIS_INFO_IOS_BEACON']).then(function (translations) {
                            $ionicPopup.alert({
                                title: translations.MSG_LOC_DIS_TITLE,
                                template: translations.MSG_LOC_DIS_INFO_IOS_BEACON
                            }).then(function (res) {
                                interruptSearching();
                            });
                        });
                    }
                });
            } else {
                checkGeolocation();
            }
        }

        function checkGeolocation() {
            if (geolocationEnable) {
                latitude = "";
                longitude = "";
                $cordovaGeolocation.getCurrentPosition({timeout: 5000, enableHighAccuracy: true}).then(function (position) {
                    latitude = position.coords.latitude.toString();
                    longitude = position.coords.longitude.toString();
                    console.log("Latitude: " + latitude + " Longitude: " + longitude);
                    startSearching();
                }, function (err) {
                    console.log("ERROR getCurrentPosition: " + JSON.stringify(err));

                    if (alertGeolocationPermission) {
                        if (Globals.PLATFORM == 'ios') {
                            if (err.code == 1 && err.message != '') {
                                // Location services are disabled
                                $translate(['MSG_LOC_DIS_TITLE', 'MSG_LOC_DIS_INFO_IOS']).then(function (translations) {
                                    $ionicPopup.alert({
                                        title: translations.MSG_LOC_DIS_TITLE,
                                        template: translations.MSG_LOC_DIS_INFO_IOS
                                    }).then(function (res) {
                                        alertGeolocationPermission = false;
                                        startSearching();
                                    });
                                });
                                return;
                            } else if (err.code == 1 && err.message == '') {
                                // Location permission denied
                                $translate(['MSG_LOC_PERM_DENY_TITLE', 'MSG_LOC_PERM_DENY_ALERT_IOS']).then(function (translations) {
                                    $ionicPopup.alert({
                                        title: translations.MSG_LOC_PERM_DENY_TITLE,
                                        template: translations.MSG_LOC_PERM_DENY_ALERT_IOS + Globals.INFO().APPLICATION_NAME + "</b>"
                                    }).then(function (res) {
                                        alertGeolocationPermission = false;
                                        startSearching();
                                    });
                                });
                                return;
                            }
                        } else if (Globals.PLATFORM == 'android') {
                            cordova.plugins.diagnostic.isLocationAvailable(function (status) {
                                console.log("Location is : " + (status ? "Available" : "Not available"));
                                if (!status) {
                                    $translate(['MSG_LOC_DIS_TITLE', 'MSG_LOC_DIS_INFO_ANDROID']).then(function (translations) {
                                        $ionicPopup.alert({
                                            title: translations.MSG_LOC_DIS_TITLE,
                                            template: translations.MSG_LOC_DIS_INFO_ANDROID
                                        }).then(function (res) {
                                            alertGeolocationPermission = false;
                                            startSearching();
                                        });
                                    });
                                } else {
                                    startSearching();
                                }

                            }, function (error) {
                                console.error("Location Available check Error: " + error);
                                startSearching();
                            });
                            return;
                        }
                    }
                    startSearching();
                });
            } else {
                startSearching();
            }
        }

        function changeMessage(text) {
            $translate(['SEARCH_TAP_REFRESH_TXT', 'SEARCHING_TXT', 'SEARCH_FOUND', 'SEARCH_NOT_FOUND', 'SEARCH_NEW_NOT_FOUND']).then(function (translations) {
                switch (text) {
                    case 'SEARCH_TAP_REFRESH_TXT':
                        $scope.infoMessage = translations.SEARCH_TAP_REFRESH_TXT;
                        $scope.infoMessageStyle = "animated fadeInDown";
                        break;
                    case 'SEARCHING_TXT':
                        $scope.infoMessage = translations.SEARCHING_TXT;
                        $scope.infoMessageStyle = "animated fadeInDown";
                        break;
                    case 'SEARCH_FOUND':
                        $scope.infoMessage = translations.SEARCH_FOUND;
                        $scope.infoMessageStyle = "animated fadeInDown";
                        break;
                    case 'SEARCH_NOT_FOUND':
                        $scope.infoMessage = translations.SEARCH_NOT_FOUND;
                        $scope.infoMessageStyle = "animated fadeInDown";
                        break;
                    case 'SEARCH_NEW_NOT_FOUND':
                        $scope.infoMessage = translations.SEARCH_NEW_NOT_FOUND;
                        $scope.infoMessageStyle = "animated fadeInDown";
                        break;
                    default:
                        $scope.infoMessageStyle = "animated fadeOutUp";
                        $timeout(function () {
                            $scope.infoMessage = '';
                        }, 500);
                        break;
                }
            });
        }

        function showRefreshButton() {
            $scope.showSpinner = false;
            $timeout(function () {
                $scope.showRefreshButton = true;
            }, 250);
            Globals.START_SEARCHING_CONTENTS = false;
        }

        function showSpinner() {
            $scope.showRefreshButton = false;
            $timeout(function () {
                $scope.showSpinner = true;
            }, 250);
        }

        function startSearching() {

            console.log("4 - Starts discovering");

            if (!simulateBeaconsEnable) {
                // Start beacon scanner
                IBeaconService.startScanning(uuid);
            } else {
                // Simulate beacons
                simulateBeacons();
            }

            // Timeout if contents not found in x seconds
            searchingTimer = $timeout(function () {

                // Ends if user has gone to see content information
                if (!searching) {
                    return;
                }

                showNoContentsFound();

            }, Globals.DISCOVER_AROUND_TIMEOUT * 1000);
        }

        // Event when beacons have been discovered
        $scope.$on("$cordovaBeacon:didRangeBeaconsInRegion", function (event, data) {
            //console.log(JSON.stringify(data));

            // Stops beacon searching
            IBeaconService.stopScanning();

            // Get nearest beacon
            var nearestBeacon = null;
            for (var i = 0; i < data.beacons.length; i++) {
                if (nearestBeacon == null || data.beacons[i].rssi > nearestBeacon.rssi) {
                    nearestBeacon = data.beacons[i];
                }
            }

            console.log("5 - Nearest beacon: " + JSON.stringify(nearestBeacon));

            var beaconUuid = "";
            var major = "";
            var minor = "";

            // Check if nearest beacon is null
            if (nearestBeacon != null) {
                beaconUuid = uuid;
                major = nearestBeacon.major.toString();
                minor = nearestBeacon.minor.toString();
            }

            // Get contents based on nearest beacon info and geolocation
            console.log("6 - Getting contents");
            if (searching) {
                PlatformService.getZoneContentsIbeaconLocation(PreferencesService.getAppId(), apiKey, beaconUuid, major, minor, latitude, longitude, "").success(function (data) {
                    if (searching) {
                        console.log("9 - Contents downloaded");
                        $timeout.cancel(searchingTimer);
                        if (data.length > 0) {
                            showContentsFound(data);
                        } else {
                            showNoContentsFound();
                        }
                    }
                }).error(function (data) {
                    $timeout.cancel(searchingTimer);
                    showNoContentsFound();
                });
            }
        });

        // Funcion to simulate beacons
        function simulateBeacons() {
            if (searching) {
                PlatformService.getZoneContentsIbeaconLocation(PreferencesService.getAppId(), apiKey, uuid, 1, Utils.randomIntFromInterval(1, 3), latitude, longitude, "").success(function (data) {
                    if (searching) {
                        console.log("9 - Contents downloaded");
                        $timeout.cancel(searchingTimer);
                        if (data.length > 0) {
                            showContentsFound(data);
                        } else {
                            showNoContentsFound();
                        }
                    }
                }).error(function (error) {
                    $timeout.cancel(searchingTimer);
                    showNoContentsFound();
                });
            }
        }

        // Function for showing discovered contents
        function showContentsFound(contents) {

            $scope.showSpinner = false;
            searching = false;

            // If autodiscovering is disable
            if (!autodiscovering) {

                // If contents are different from last search show message and show new contents
                if (Globals.LAST_DISCOVERED_CONTENTS != JSON.stringify(contents)) {
                    changeMessage("SEARCH_FOUND");
                    $timeout(function () {
                        changeMessage("");
                        $scope.contents = {};
                        $timeout(function () {
                            $scope.contents = contents;
                            Globals.LAST_DISCOVERED_CONTENTS = JSON.stringify(contents);
                            showRefreshButton();
                        }, 500);

                    }, 1200);

                } else {
                    changeMessage("SEARCH_NEW_NOT_FOUND");
                    $timeout(function () {
                        changeMessage("");
                        showRefreshButton();
                    }, 1200);
                }

            } else {

                // If contents are different from last search show message and show new contents
                if (Globals.LAST_DISCOVERED_CONTENTS != JSON.stringify(contents)) {
                    $scope.contents = {};
                    $timeout(function () {
                        $scope.contents = contents;
                        Globals.LAST_DISCOVERED_CONTENTS = JSON.stringify(contents);
                    }, 500);
                }

                // Reset no contents discovered times 
                noContentsDiscoveredCurrentTimes = 0;

                // Starts discovering again before x seconds
                $scope.showProgressbar = true;
                $scope.progressbarMaxValue = autodiscoveringTimer * 1000;
                $scope.progressbarValue = autodiscoveringTimer * 1000;
                runningAutodiscoveringProgressbar = true;
                runProgressbar();
            }
        }

        // Function for showing no contents found info
        function showNoContentsFound() {

            $scope.showSpinner = false;
            searching = false;

            // If autodiscovering is disable
            if (!autodiscovering) {

                changeMessage("SEARCH_NOT_FOUND");
                $timeout(function () {
                    changeMessage("");
                    $scope.contents = [];
                    Globals.LAST_DISCOVERED_CONTENTS = JSON.stringify($scope.contents);
                }, 1200);

                showRefreshButton();

            } else {

                // Increment counter
                noContentsDiscoveredCurrentTimes++;

                // If contents not found in x times, remove all from view
                if (Globals.NO_CONTENTS_DISCOVERED_TIMES == noContentsDiscoveredCurrentTimes) {
                    noContentsDiscoveredCurrentTimes = 0;
                    $scope.contents = [];
                    Globals.LAST_DISCOVERED_CONTENTS = JSON.stringify($scope.contents);
                }

                // Starts discovering again before x seconds
                $scope.showProgressbar = true;
                $scope.progressbarMaxValue = autodiscoveringTimer * 1000;
                $scope.progressbarValue = autodiscoveringTimer * 1000;
                runningAutodiscoveringProgressbar = true;
                runProgressbar();
            }
        }

        // Function to run progressbar and run autodiscovering again
        function runProgressbar() {
            if (runningAutodiscoveringProgressbar) {
                if ($scope.progressbarValue > 0) {
                    $timeout(function () {
                        $scope.progressbarValue = $scope.progressbarValue - 50;
                        runProgressbar();
                    }, 50);
                } else {
                    console.log("10 - Showing contents");
                    $scope.showProgressbar = false;
                    $scope.showSpinner = true;
                    runningAutodiscoveringProgressbar = false;
                    checkBle();
                }
            }
        }

        // Event when users tap in contents
        $scope.showContent = function (content) {

            searching = false;
            runningAutodiscoveringProgressbar = false;
            $timeout.cancel(searchingTimer);
            IBeaconService.stopScanning();

            // Register user interaction: details
            PlatformService.logUserInteraction(PreferencesService.getAppId(), apiKey, "details", content.id, "").success(function (data) {
                console.log("SUCCESS: " + data);
            }).error(function (error) {
                console.log("ERROR: " + error);
            });

            // Show content
            //beaconRangeEvent();
            Globals.SELECTED_CONTENT = content;
            Globals.CONTENTVIEW_OPENED_FROM = "search";
            $state.go('app.contentView');
        };

        // To convert external url's as a trusted
        $scope.trustSrc = function (src) {
            return $sce.trustAsResourceUrl(src);
        };

        // Event when view has been destroyed
        $scope.$on('$destroy', function () {

            if (!Globals.START_SEARCHING_CONTENTS) {
                // Unregister beacon range event
                console.log("SearchContentsController Destroyed");
                searching = false;
                $timeout.cancel(searchingTimer);
                runningAutodiscoveringProgressbar = false;
                IBeaconService.stopScanning();
                //beaconRangeEvent();
            }

        });
    }]);