/* global angular */
angular.module('starter.controllers').controller('SettingsController', ['$scope', '$rootScope', '$translate', '$state', '$ionicPopup', '$ionicPlatform', '$ionicLoading', '$timeout', '$cordovaGeolocation', 'PreferencesService', 'Globals', 'PlatformService', 'PushNotificationsService',
    function ($scope, $rootScope, $translate, $state, $ionicPopup, $ionicPlatform, $ionicLoading, $timeout, $cordovaGeolocation, PreferencesService, Globals, PlatformService, PushNotificationsService) {

        $scope.data = {};

        $ionicPlatform.ready(function () {

             if (Globals.PLATFORM == 'android') {
             $rootScope.myService = cordova.plugins.easyContextService;
             $rootScope.myService.getStatus(function (r) {
             console.log("MyService: " + JSON.stringify(r));
             }, function (e) {
             console.log("MyService Error: " + JSON.stringify(e));
             });
             }

            // Gets if application is demo app or not
            if (Globals.IS_DEMO_APP) {
                $scope.data.platform = Globals.PLATFORM;
                $scope.data.isDemoApp = true;
                $scope.data.apiKey = PreferencesService.getApiKey() == null ? Globals.API_KEY : PreferencesService.getApiKey();
                $scope.data.uuid = PreferencesService.getUuid() == null ? Globals.UUID : PreferencesService.getUuid();
                $scope.data.simulateBeacons = PreferencesService.getBeaconsSimulation();

                $scope.data.autodiscoveringTimer = PreferencesService.getAutoDiscoveringTimer() == null ? 15 : parseInt(PreferencesService.getAutoDiscoveringTimer());
            }

            // Gets other preferences
            $scope.data.geolocation = PreferencesService.getGeolocation();
            $scope.data.notifications = PreferencesService.getPushNotificationsEnable();
            $scope.data.automaticDiscovering = PreferencesService.getAutoDiscovering();
            $scope.data.autodiscoveringEnable = PreferencesService.getAutoDiscovering();

            $scope.data.backgroundMode = PreferencesService.getBackgroundMode();
            $scope.data.backgroundBatterySaverMode = PreferencesService.getBackgroundBatterySaverMode();
        });

        // Function to enable/disbale background service only for Android
        function toggleAndroidBackgroundMode(value) {
            if (value) {
                // Enable background mode
                cordova.plugins.backgroundMode.enable();

                // Starting service to register for boot start
                $rootScope.myService.startService(function (r) {
                    console.log("Start MyService: " + JSON.stringify(r));

                    // Configuring the time for waiting to start beacon search
                    timerWait = Globals.BACKGROUND_BEACON_SEARCH_DURATION;
                    if ($scope.data.backgroundBatterySaverMode) {
                        // In battery saver mode is about 5 + 30 = 35 seconds
                        timerWait += Globals.BACKGROUND_BEACON_SEARCH_WAIT_BATTERY_SAVER;
                    } else {
                        // By default is about 5 + 5 = 10 seconds
                        timerWait += Globals.BACKGROUND_BEACON_SEARCH_WAIT_DEFAULT;
                    }

                    // Setting service configurations
                    var config = {
                        "BeaconUUID": PreferencesService.getUuid(),
                        "ENABLED": false,
                        "APP_ID": PreferencesService.getAppId(),
                        "API_KEY": PreferencesService.getApiKey(),
                        "REG_TOKEN": PreferencesService.getPushToken(),
                        "TIMER_MILLISECONDS": (timerWait * 1000),
                        "USE_GEO": $scope.data.geolocation,
                        "SEARCH_DURATION": (Globals.BACKGROUND_BEACON_SEARCH_DURATION * 1000)
                    };
                    console.log(JSON.stringify(config));
                    $rootScope.myService.setConfiguration(config, function (setconfig) {
                        console.log("Set Config Response: " + JSON.stringify(setconfig));
                    }, function (errorconfig) {
                        console.log("Set Config Error: " + JSON.stringify(errorconfig));
                    });

                    // Setting timer of the service
                    $rootScope.myService.enableTimer(timerWait * 1000, function (timer_response) {
                        console.log("Enable Timer MyService: " + JSON.stringify(timer_response));

                        // Register for Boot start
                        $rootScope.myService.registerForBootStart(function (bootstart_response) {
                            console.log("Enable BootStart MyService: " + JSON.stringify(bootstart_response));

                            // Stopping the service itself, but keeping the BootStart active
                            $rootScope.myService.stopService(function (stop_service) {
                                console.log("Stop MyService: " + JSON.stringify(stop_service));
                            }, function (e) {
                                console.log("Stop MyService Error: " + JSON.stringify(e));
                            });

                        }, function (bootstart_error) {
                            console.log("Enable BootStart Error MyService: " + JSON.stringify(bootstart_error));
                        });

                    }, function (timer_error) {
                        console.log("Enable Timer Error MyService: " + JSON.stringify(timer_error));
                    });

                }, function (e) {
                    console.log("Start MyService Error: " + JSON.stringify(e));
                });
            } else {
                // Disabling background mode
                cordova.plugins.backgroundMode.disable();

                // Unregister from BootStart
                $rootScope.myService.deregisterForBootStart(function (stopboot) {
                    console.log("Stop from BootStart MyService: " + JSON.stringify(stopboot));
                }, function (stopboot_error) {
                    console.log("Stop BootStart MyService Error: " + JSON.stringify(stopboot_error));
                });
            }
        }

        $scope.onInfoDialog = function (item) {

            $translate([
                'SETTINGS_INFO_DIALOG_BACKGROUND_TEMPLATE',
                'SETTINGS_INFO_DIALOG_AUTODISCOVERING_TEMPLATE',
                'SETTINGS_INFO_DIALOG_TITLE'
            ]).then(function (translations) {
                var t;
                switch (item) {
                    case "background":
                        t = translations.SETTINGS_INFO_DIALOG_BACKGROUND_TEMPLATE;
                        break;
                    case "auto_discovering":
                        t = translations.SETTINGS_INFO_DIALOG_AUTODISCOVERING_TEMPLATE;
                        break;
                }
                $ionicPopup.alert({
                    title: translations.SETTINGS_INFO_DIALOG_TITLE,
                    template: t
                });
            });

        };

        // Evento al pulsar toggle de background battery saver mode
        $scope.onBackgroundBatterySaverMode = function () {
            var value = $scope.data.backgroundBatterySaverMode;
            console.log("Background Battery Saver Mode: " + value);
            PreferencesService.setBackgroundBatterySaverMode(value);

            // Reconfigure the background mode
            $scope.onBackgroundMode();
        };

        // Evento al pulsar toggle de modo background
        $scope.onBackgroundMode = function () {
            var value = $scope.data.backgroundMode;

            if (value) {
                // if push notification is disabled we can't start background mode
                if ($scope.data.notifications) {
                    console.log("Background Mode: " + value);
                    PreferencesService.setBackgroundMode(value);
                    toggleAndroidBackgroundMode(value);
                } else {
                    // if notification is disabled ask to enable and then enable background mode
                    $translate([
                        'SETTINGS_BACKGROUND_NEEDS_PUSH',
                        'SETTINGS_BACKGROUND_NEEDS_PUSH_TEMP',
                        'BUTTON_NO',
                        'BUTTON_YES'
                    ]).then(function (translations) {
                        //prevent changing API-KEY for demo account
                        $ionicPopup.confirm({
                            title: translations.SETTINGS_BACKGROUND_NEEDS_PUSH,
                            template: translations.SETTINGS_BACKGROUND_NEEDS_PUSH_TEMP,
                            buttons: [
                                {
                                    text: translations.BUTTON_NO,
                                    type: 'button-default',
                                    onTap: function (e) {
                                        console.log('User is not agree');
                                        $scope.data.backgroundMode = false;
                                    }
                                },
                                {
                                    text: translations.BUTTON_YES,
                                    type: 'button-assertive',
                                    onTap: function (e) {
                                        // user agrees to enable push notifications
                                        $scope.data.notifications = true;
                                        $scope.onNotificationsChange();
                                        console.log("Background Mode: " + value);
                                        PreferencesService.setBackgroundMode(value);
                                        toggleAndroidBackgroundMode(value);
                                    }
                                }
                            ]
                        });
                    });
                }
            } else {
                console.log("Background Mode: " + value);
                PreferencesService.setBackgroundMode(value);
                toggleAndroidBackgroundMode(value);
            }
        };

        // Evento al pulsar checkbox de simulacion de beacons
        $scope.onBeaconsSimulationChange = function () {
            var value = $scope.data.simulateBeacons;
            console.log(value);
            PreferencesService.setBeaconsSimulation(value);
        };

        // Evento al pulsar checkbox de notificaciones
        $scope.onNotificationsChange = function () {
            var value = $scope.data.notifications;

            if (value) {
                console.log("Push notification enabled: " + value);
                PreferencesService.setPushNotificationsEnable(value);
                // Register push notifications if not registered yet
                PushNotificationsService.registerPushNotifications();
            } else {
                if ($scope.data.backgroundMode) {
                    // If the background mode is enabled show message that background can't work with disabled push notifications
                    $translate([
                        'SETTINGS_BACKGROUND_NEEDS_PUSH',
                        'SETTINGS_BACKGROUND_NEEDS_PUSH_TEMP_2',
                        'BUTTON_NO',
                        'BUTTON_YES'
                    ]).then(function (translations) {
                        //prevent changing API-KEY for demo account
                        $ionicPopup.confirm({
                            title: translations.SETTINGS_BACKGROUND_NEEDS_PUSH,
                            template: translations.SETTINGS_BACKGROUND_NEEDS_PUSH_TEMP_2,
                            buttons: [
                                {
                                    text: translations.BUTTON_NO,
                                    type: 'button-default',
                                    onTap: function (e) {
                                        $scope.data.notifications = true;
                                    }
                                },
                                {
                                    text: translations.BUTTON_YES,
                                    type: 'button-assertive',
                                    onTap: function (e) {
                                        // Unregister push notifications if not registered yet
                                        PushNotificationsService.unregisterPushNotifications();
                                        console.log("Push notification enabled: " + value);
                                        PreferencesService.setPushNotificationsEnable(value);

                                        // Disabling background mode
                                        $scope.data.backgroundMode = false;
                                        $scope.onBackgroundMode();
                                    }
                                }
                            ]
                        });
                    });
                } else {
                    // Unregister push notifications if not registered yet
                    PushNotificationsService.unregisterPushNotifications();
                    console.log("Push notification enabled: " + value);
                    PreferencesService.setPushNotificationsEnable(value);
                }
            }
        };

        function updateScopeGeolocation(value) {
            $scope.data.geolocation = value;
            PreferencesService.setGeolocation(value);
        }

        // Evento al pulsar checkbox de geolocation
        $scope.onGeolocation = function () {
            var value = $scope.data.geolocation;
            console.log("Use geolocation to search contents: " + value);

            if (value) {
                // If mobile platform is android and platform is minor than 6.0
                if (Globals.PLATFORM == 'android') {
                    // Check Device Location if it is Available
                    cordova.plugins.diagnostic.isLocationAvailable(function (status) {
                        console.log("Location is : " + (status ? "Available" : "Not available"));
                        // Location is Off
                        if (!status) {
                            //Ask user to enable it
                            $translate([
                                'BUTTON_CANCEL',
                                'BUTTON_ACCEPT',
                                'MSG_LOC_DIS_TITLE',
                                'MSG_LOC_DIS_TEMPLATE'
                            ]).then(function (translations) {
                                $ionicPopup.confirm({
                                    title: translations.MSG_LOC_DIS_TITLE,
                                    template: translations.MSG_LOC_DIS_TEMPLATE,
                                    buttons: [
                                        {
                                            text: translations.BUTTON_CANCEL,
                                            type: 'button-default',
                                            onTap: function (e) {
                                                console.log('Location Activation Canceled');
                                                updateScopeGeolocation(false);
                                            }
                                        },
                                        {
                                            text: translations.BUTTON_ACCEPT,
                                            type: 'button-positive',
                                            onTap: function (e) {

                                                cordova.plugins.locationAccuracy.request(function () {
                                                    console.log("Location Accuracy: Request successful");
                                                    getLocation();
                                                }, function (error) {
                                                    console.error("Location Accuracy Error: Request failed: " + JSON.stringify(error));

                                                    updateScopeGeolocation(false);
                                                    $scope.$apply();

                                                    if (error) {
                                                        console.error("Location Accuracy Request Error: code=" + error.code + "; error message=" + error.message);
                                                        if (error.code !== cordova.plugins.locationAccuracy.ERROR_USER_DISAGREED) {
                                                            //if programatically enabling location failed, show location settings window
                                                            cordova.plugins.diagnostic.switchToLocationSettings();
                                                        }
                                                    }
                                                }, cordova.plugins.locationAccuracy.REQUEST_PRIORITY_HIGH_ACCURACY);
                                            }
                                        }
                                    ]
                                });
                            });

                        } else {
                            //Location is available, continue
                            getLocation();
                        }
                    }, function (error) {
                        console.error("Location Available check Error: " + error);
                        updateScopeGeolocation(false);
                        $ionicPopup.alert({
                            title: "Location checking eror",
                            template: "The app was not able to check your location settings."
                        });
                    });
                } else {
                    // ios: Continue
                    getLocation();
                }
            } else {
                updateScopeGeolocation(false);
            }
        };

        function getLocation() {
            $cordovaGeolocation.getCurrentPosition({timeout: 10000, enableHighAccuracy: true}).then(function (position) {
                var latitude = position.coords.latitude;
                var longitude = position.coords.longitude;
                console.log("Latitude: " + latitude + " Longitude: " + longitude);
                updateScopeGeolocation(true);

            }, function (err) {
                console.log("ERROR getCurrentPosition: " + JSON.stringify(err));
                updateScopeGeolocation(false);

                if (Globals.PLATFORM == 'ios') {

                    if (err.code == 1 && err.message != '') {
                        // Location services are disabled
                        $translate(['MSG_LOC_DIS_TITLE', 'MSG_LOC_DIS_INFO_IOS']).then(function (translations) {
                            $ionicPopup.alert({
                                title: translations.MSG_LOC_DIS_TITLE,
                                template: translations.MSG_LOC_DIS_INFO_IOS
                            }).then(function (res) {});
                        });

                    } else if (err.code == 1 && err.message == '') {
                        // Location permission denied
                        $translate(['MSG_LOC_PERM_DENY_TITLE', 'MSG_LOC_PERM_DENY_ALERT_IOS']).then(function (translations) {
                            $ionicPopup.alert({
                                title: translations.MSG_LOC_PERM_DENY_TITLE,
                                template: translations.MSG_LOC_PERM_DENY_ALERT_IOS + Globals.INFO().APPLICATION_NAME + "</b>"
                            }).then(function (res) {});
                        });
                    }

                } else if (Globals.PLATFORM == 'android') {

                    if (err.code == 1 && err.message == 'Illegal Access') {
                        // Location services are disabled
                        $translate(['MSG_LOC_DENIED_TITLE', 'MSG_LOC_DENIED_TEMPLATE']).then(function (translations) {
                            $ionicPopup.alert({
                                title: translations.MSG_LOC_DENIED_TITLE,
                                template: translations.MSG_LOC_DENIED_TEMPLATE
                            }).then(function (res) {});
                        });
                    }
                }
            });
        }

        // Evento al pulsar checkbox de auto descubrimiento
        $scope.onAutoDiscovering = function () {
            var value = $scope.data.automaticDiscovering;
            console.log(value);
            PreferencesService.setAutoDiscovering(value);
        };

        // Evento al pulsar sobre la API-KEY
        $scope.onApiKeyClick = function () {
            $translate([
                'SETTINGS_CANT_CHANGE_API_KEY_TITLE',
                'SETTINGS_CANT_CHANGE_APIKEY_UUID_TEMPLATE',
                'WELCOME_INCORRECT_APIKEY_FORMAT',
                'SETTINGS_APIKEY_CHECK_ERROR',
                'SETTINGS_APIKEY_NOTACTIVATED',
                'WELCOME_INCORRECT_API',
                'APP_LOGOUT_TITLE',
                'BUTTON_CANCEL',
                'BUTTON_ACCEPT'
            ]).then(function (translations) {
                if ($scope.data.apiKey == Globals.API_KEY_DEMO_ACCOUNT) {
                    //prevent changing API-KEY for demo account
                    $ionicPopup.confirm({
                        title: translations.SETTINGS_CANT_CHANGE_API_KEY_TITLE,
                        template: translations.SETTINGS_CANT_CHANGE_APIKEY_UUID_TEMPLATE,
                        buttons: [
                            {
                                text: translations.BUTTON_CANCEL,
                                type: 'button-default',
                                onTap: function (e) {
                                    console.log('You are not sure');
                                }
                            },
                            {
                                text: translations.APP_LOGOUT_TITLE,
                                type: 'button-assertive',
                                onTap: function (e) {
                                    PreferencesService.logout();
                                    Globals.LAST_DISCOVERED_CONTENTS = {};
                                    Globals.SELECTED_CONTENT = {};
                                    $state.go('welcome');
                                }
                            }
                        ]
                    });

                } else {
                    var apiKeyPopup;
                    apiKeyPopup = $ionicPopup.show({
                        template: '<input type="text" style="text-align:center" ng-model="data.apiKey" mask="****-****-****-****">',
                        title: 'EasyContext API-KEY',
                        subTitle: 'Format: XXXX-XXXX-XXXX-XXXX',
                        scope: $scope,
                        buttons: [
                            {text: translations.BUTTON_CANCEL, onTap: function (e) {
                                    $scope.data.apiKey = PreferencesService.getApiKey() == null ? Globals.API_KEY : PreferencesService.getApiKey();
                                }
                            },
                            {text: '<b>' + translations.BUTTON_ACCEPT + '</b>', type: 'button-positive',
                                onTap: function (e) {
                                    //prevent closing the propmt
                                    e.preventDefault();
                                    var errorText = "";
                                    //checking if the introduced api key have correct format
                                    if (isValidApiKeyFormat($scope.data.apiKey)) {
                                        //validating api key from the platform
                                        PlatformService.checkApiKey($scope.data.apiKey).then(function (serverCheckApiKey) {
                                            switch (serverCheckApiKey.response) {
                                                case "incorrect":
                                                    errorText = translations.WELCOME_INCORRECT_API;
                                                    break;
                                                case "inactive":
                                                    errorText = translations.SETTINGS_APIKEY_NOTACTIVATED;
                                                    break;
                                                case "correct":
                                                    PreferencesService.setApiKey($scope.data.apiKey);
                                                    PreferencesService.setAccountAccess(serverCheckApiKey.access);
                                                    break;
                                                default:
                                                    errorText = translations.SETTINGS_APIKEY_CHECK_ERROR;
                                                    break;
                                            }
                                        }).catch(function (data, status) {
                                            errorText = translations.SETTINGS_APIKEY_CHECK_ERROR;
                                        }).finally(function () {
                                            if (errorText !== "") {
                                                $ionicLoading.show({
                                                    template: '<p><i class="ion-close-round"></p></i><p>' + errorText + '</p>'
                                                });
                                                $timeout(function () {
                                                    $ionicLoading.hide();
                                                }, 1500);
                                            } else {
                                                apiKeyPopup.close();
                                            }
                                        });

                                    } else {
                                        errorText = translations.WELCOME_INCORRECT_APIKEY_FORMAT;
                                        $ionicLoading.show({
                                            template: '<p><i class="ion-close-round"></p></i><p>' + errorText + '</p>'
                                        });
                                        $timeout(function () {
                                            $ionicLoading.hide();
                                        }, 1500);
                                    }
                                }
                            }
                        ]
                    });
                }
            });
        };

        // Funcion para validar el API KEY Format
        function isValidApiKeyFormat(apikey) {
            return apikey.length === 19 && apikey.match("[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}");
        }

        // Evento al pulsar sobre la UUID
        $scope.onUuidClick = function () {

            $translate([
                'SETTINGS_CANT_CHANGE_UUID_TITLE',
                'SETTINGS_INCORRECT_UUID_FORMAT',
                'SETTINGS_CANT_CHANGE_APIKEY_UUID_TEMPLATE',
                'APP_LOGOUT_TITLE',
                'BUTTON_CANCEL',
                'BUTTON_ACCEPT',
                'SETTINGS_CANT_CHANGE_UUID_IOS_TITLE',
                'SETTINGS_CANT_CHANGE_UUID_IOS_TEMPLATE'
            ]).then(function (translations) {

                if (Globals.PLATFORM == 'ios') {
                    $ionicPopup.alert({
                        title: translations.SETTINGS_CANT_CHANGE_UUID_IOS_TITLE,
                        template: translations.SETTINGS_CANT_CHANGE_UUID_IOS_TEMPLATE
                    }).then(function (res) {});

                } else if (Globals.PLATFORM == 'android') {

                    if ($scope.data.apiKey == Globals.API_KEY_DEMO_ACCOUNT) {
                        //prevent changing UUID for demo account
                        $ionicPopup.confirm({
                            title: translations.SETTINGS_CANT_CHANGE_UUID_TITLE,
                            template: translations.SETTINGS_CANT_CHANGE_APIKEY_UUID_TEMPLATE,
                            buttons: [
                                {
                                    text: translations.BUTTON_CANCEL,
                                    type: 'button-default',
                                    onTap: function (e) {
                                        console.log('You are not sure');
                                    }
                                },
                                {
                                    text: translations.APP_LOGOUT_TITLE,
                                    type: 'button-assertive',
                                    onTap: function (e) {
                                        PreferencesService.logout();
                                        Globals.LAST_DISCOVERED_CONTENTS = {};
                                        Globals.SELECTED_CONTENT = {};
                                        $state.go('welcome');
                                    }
                                }
                            ]
                        });
                    } else {
                        $ionicPopup.show({
                            template: '<input type="text" ng-model="data.uuid" mask="********-****-****-****-************" capitalize>',
                            title: 'UUID',
                            subTitle: 'Format: XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX',
                            scope: $scope,
                            buttons: [
                                {text: translations.BUTTON_CANCEL, onTap: function (e) {
                                        $scope.data.uuid = PreferencesService.getUuid() == null ? Globals.UUID : PreferencesService.getUuid();
                                    }
                                },
                                {text: '<b>Accept</b>', type: 'button-positive',
                                    onTap: function (e) {
                                        if (isValidUuid($scope.data.uuid)) {
                                            PreferencesService.setUuid($scope.data.uuid);
                                        } else {
                                            $scope.data.uuid = PreferencesService.getUuid() == null ? Globals.UUID : PreferencesService.getUuid();
                                            $ionicLoading.show({
                                                template: '<p><i class="ion-close-round"></p></i><p>' + translations.SETTINGS_INCORRECT_UUID_FORMAT + '</p>'
                                            });
                                            $timeout(function () {
                                                $ionicLoading.hide();
                                            }, 1500);
                                        }
                                    }
                                }
                            ]
                        });
                    }
                }
            });
        };

        // Funcion para validar la UUID
        function isValidUuid(uuid) {
            return uuid.length === 36 && uuid.match("[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}");
        }

        // Evento al pulsar sobre el temporizador de autodescubrimiento
        $scope.onAutomaticDiscoveringTimerClick = function () {

            $translate([
                'SETTINGS_AUT_DISCOVERING_TIMER_TITLE',
                'SETTINGS_AUT_DISCOVERING_TIMER_TEMPLATE',
                'SETTINGS_INCORRECT_AUTODISCOVERING_TIMER_VALUE_TITLE',
                'SETTINGS_INCORRECT_AUTODISCOVERING_TIMER_VALUE',
                'SETTINGS_INCORRECT_TIMER_FORMAT',
                'BUTTON_CANCEL',
                'BUTTON_ACCEPT',
            ]).then(function (translations) {

                $ionicPopup.show({
                    template: '<input type="number" style="text-align:center" ng-model="data.autodiscoveringTimer">',
                    title: translations.SETTINGS_AUT_DISCOVERING_TIMER_TITLE,
                    subTitle: translations.SETTINGS_AUT_DISCOVERING_TIMER_TEMPLATE,
                    scope: $scope,
                    buttons: [
                        {text: translations.BUTTON_CANCEL, onTap: function (e) {
                                $scope.data.autodiscoveringTimer = PreferencesService.getAutoDiscoveringTimer() == null ? 15 : parseInt(PreferencesService.getAutoDiscoveringTimer());
                            }
                        },
                        {text: '<b>' + translations.BUTTON_ACCEPT + '</b>', type: 'button-positive',
                            onTap: function (e) {

                                // Checks if input is a number
                                if (isNaN($scope.data.autodiscoveringTimer)) {
                                    $ionicLoading.show({
                                        template: '<p><i class="ion-close-round"></p></i><p>' + translations.SETTINGS_INCORRECT_TIMER_FORMAT + '</p>'
                                    });
                                    $timeout(function () {
                                        $ionicLoading.hide();
                                    }, 1500);
                                    $scope.data.autodiscoveringTimer = PreferencesService.getAutoDiscoveringTimer() == null ? 15 : parseInt(PreferencesService.getAutoDiscoveringTimer());
                                    return;
                                }

                                // Checks if autodiscovering timer is in raange
                                if ($scope.data.autodiscoveringTimer < 10 || $scope.data.autodiscoveringTimer > 300) {
                                    $ionicPopup.alert({
                                        title: translations.SETTINGS_INCORRECT_AUTODISCOVERING_TIMER_VALUE_TITLE,
                                        template: translations.SETTINGS_INCORRECT_AUTODISCOVERING_TIMER_VALUE
                                    }).then(function (res) {});
                                    $scope.data.autodiscoveringTimer = PreferencesService.getAutoDiscoveringTimer() == null ? 15 : parseInt(PreferencesService.getAutoDiscoveringTimer());
                                    return;
                                }

                                // Sets new autodiscovering timer
                                PreferencesService.setAutoDiscoveringTimer($scope.data.autodiscoveringTimer);
                            }
                        }
                    ]
                });
            });
        };

    }]);