/* global angular */
angular.module('starter.controllers').controller('UserProfileController', ['$scope', '$ionicPopup', '$ionicLoading', '$translate', '$ionicPlatform', '$state', 'Globals', 'PreferencesService', 'PlatformService', 'Utils',
    function ($scope, $ionicPopup, $ionicLoading, $translate, $ionicPlatform, $state, Globals, PreferencesService, PlatformService, Utils) {
        $scope.editMode = false;

        $scope.button = {};
        $scope.button.female = {};
        $scope.button.male = {};

        $scope.userData = {};

        var loginData;

        $ionicPlatform.ready(function () {

            // Getting company lables
            $scope.lables = Globals.INFO().LABLES_ID;
            console.log($scope.lables);

            // Getting localy stored profile data
            loginData = PreferencesService.getLoginData();
            $scope.userData.username = loginData.USERNAME_TAG;

            $scope.userData.birthdate_value = loginData.BIRTHDATE_TAG;
            if (!($scope.userData.birthdate_value === null || $scope.userData.birthdate_value === "null" ||
                    $scope.userData.birthdate_value === "undefined" || $scope.userData.birthdate_value === undefined)) {
                $scope.userData.birthdate_value = new Date($scope.userData.birthdate_value);
                $scope.userData.birthdate = Utils.formatDate(new Date($scope.userData.birthdate_value));
            }

            $scope.userData.gender = loginData.GENDER_TAG;
            if (!($scope.userData.gender === null || $scope.userData.gender === "null" ||
                    $scope.userData.gender === "undefined" || $scope.userData.gender === undefined)) {
                if ($scope.userData.gender === 'female') {
                    $scope.button.female.clicked = true;
                } else if ($scope.userData.gender === 'male') {
                    $scope.button.male.clicked = true;
                }
            }

            $scope.userData.lables = loginData.LABELS_TAG;

            if ($scope.userData.lables === null || $scope.userData.lables === "null" || $scope.userData.lables === "undefined" || $scope.userData.lables === undefined) {
                console.log("No Lables");
                $scope.userData.lables = [];
            } else {
                console.log($scope.userData.lables);
            }

            console.log(loginData);
        });


        // Function to start editing or finish saving changes on profile
        $scope.editSave = function () {

            if ($scope.editMode) {
                var today = new Date();
                today.setHours(0, 0, 0, 0);
                var birthdate = new Date($scope.userData.birthdate_value);
                // Birthdate validation
                if (birthdate.getTime() < today.getTime() && $scope.userData.birthdate_value !== null) {
                    console.log("Saving user profile data");
                    $ionicLoading.show({
                        template: '<ion-spinner icon="android"></ion-spinner>'
                    });
                    // Saving on platform
                    PlatformService.updateUserData(
                            loginData.APP_ID_TAG,
                            PreferencesService.getApiKey(),
                            $scope.userData.birthdate,
                            $scope.userData.gender,
                            $scope.userData.username,
                            $scope.userData.lables.toString()
                            ).success(function (data) {
                        // Saved on platform
                        console.log("User profile updated on platform: " + JSON.stringify(data));

                        // Saving changes locally
                        PreferencesService.updateProfileData(
                                $scope.userData.birthdate,
                                $scope.userData.gender,
                                $scope.userData.lables
                                );
                        $ionicLoading.hide();
                        $scope.editMode = !$scope.editMode;
                        $translate([
                            'PROFILE_UPDATE',
                            'PROFILE_UPDATE_SUCCESS']).then(function (translations) {
                            $ionicPopup.alert({
                                title: translations.PROFILE_UPDATE,
                                template: translations.PROFILE_UPDATE_SUCCESS
                            }).then(function (res) {

                            });
                        });

                    }).error(function (error_data) {
                        console.log("Error on updating user profile on platform: " + JSON.stringify(error_data));
                        $ionicLoading.hide();
                        $translate([
                            'PROFILE_UPDATE',
                            'PROFILE_UPDATE_ERROR']).then(function (translations) {
                            $ionicPopup.alert({
                                title: translations.PROFILE_UPDATE,
                                template: translations.PROFILE_UPDATE_ERROR
                            }).then(function (res) { });
                        });
                    });
                } else {
                    console.log("Birthdate validation error");
                    $translate([
                        'REGISTER_BIRTHDATE_ERROR_WARNING_TITLE',
                        'MSG_ERR_REG_REQUIRED',
                        'REGISTER_BIRTHDATE_ERROR_WARNING_MESSAGE']).then(function (translations) {
                        $ionicPopup.alert({
                            title: translations.REGISTER_BIRTHDATE_ERROR_WARNING_TITLE,
                            template: ($scope.userData.birthdate_value === null) ? translations.MSG_ERR_REG_REQUIRED : translations.REGISTER_BIRTHDATE_ERROR_WARNING_MESSAGE
                        });
                    });
                    return;
                }
            } else {
                $scope.editMode = !$scope.editMode;
            }
        };

        // Function to change birthdate to right format
        $scope.onBirthdateChange = function (val) {
            if (val === null | val === "null") {
                $scope.userData.birthdate = null;
            } else {
                $scope.userData.birthdate = Utils.formatDate(new Date(val));
            }
        };

        // Function to detect and process gender change 
        $scope.onGenderChange = function (button) {
            if ($scope.editMode) {
                $scope.button.male.clicked = false;
                $scope.button.female.clicked = false;
                button.clicked = true;

                if ($scope.button.female.clicked) {
                    $scope.userData.gender = 'female';

                } else if ($scope.button.male.clicked) {
                    $scope.userData.gender = 'male';
                }
            }
        };

        // Helper function to identify lable type to translte
        $scope.getLabelType = function (type) {
            return "LABLE_" + type.toUpperCase();
        };

        // Adding and removing labels from profile
        $scope.processLableChange = function (lable, toadd) {
            var indexOfLabel = $.inArray(lable, $scope.userData.lables);
            if (toadd) {
                if (indexOfLabel === -1) {
                    // add if not exists
                    $scope.userData.lables.push(lable);
                    console.log("Label " + lable + " is added to profile.");
                } else {
                    console.log("Label " + lable + " is already in profile.");
                }
            } else {
                //remove label
                $scope.userData.lables.splice(indexOfLabel, 1);
                console.log("Label " + lable + " was removed from profile.");
            }
        };

        // Check if lable already exists in profile
        $scope.lableExistInProfile = function (lable) {
            return  $.inArray(lable, $scope.userData.lables) !== -1;
        };
    }]);