/* global angular */
angular.module('starter.controllers').controller('WelcomeController', ['$scope', '$cordovaDevice', '$translate', '$state', '$ionicPopup', '$ionicPlatform', '$ionicLoading', '$timeout', 'Globals', 'PreferencesService', 'PlatformService', '$cordovaToast',
    function ($scope, $cordovaDevice, $translate, $state, $ionicPopup, $ionicPlatform, $ionicLoading, $timeout, Globals, PreferencesService, PlatformService, $cordovaToast) {

        $scope.data = {};

        $scope.contact = Globals.INFO().CONTACT_DATA;

        $ionicPlatform.ready(function () {
            console.log("READY WELCOME");
            // Evento al pulsar el boton de volver atras
            $ionicPlatform.registerBackButtonAction(function (event) {
                console.log("Back device button clicked");
                navigator.app.exitApp();
            }, 100);
        });

        // Evento antes de cargar la vista de la pantalla de bienvenida
        $scope.$on("$ionicView.beforeEnter", function (event, data) {
            $scope.data.apiKey = "";
        });

        // Evento al introducir el API-KEY
        $scope.enterApiKey = function () {
            // Se comprueba si el formato del API-KEY es correct
            $translate([
                'WELCOME_INCORRECT_API',
                'WELCOME_ACCOUNT_CONFIRM_TITLE',
                'WELCOME_ACCOUNT_CONFIRM_TEXT',
                'WELCOME_INCORRECT_API_CHECK',
                'WELCOME_INCORRECT_APIKEY_FORMAT'
            ]).then(function (translations) {
                if (isValidApiKey($scope.data.apiKey)) {
                    // Se comprueba si existe el API-KEY en la plataforma y esta activado
                    PlatformService.checkApiKey($scope.data.apiKey).success(function (data) {
                        if (data.response === 'incorrect') {
                            // API-KEY incorrecta
                            showToastMessage(translations.WELCOME_INCORRECT_API);
                        } else if (data.response === 'inactive') {
                            // Cuenta no activada
                            $ionicPopup.confirm({
                                title: translations.WELCOME_ACCOUNT_CONFIRM_TITLE,
                                template: translations.WELCOME_ACCOUNT_CONFIRM_TEXT
                            }).then(function (res) {});
                        } else if (data.response === 'correct') {
                            // API-KEY correcta
                            //TODO: check the access to show or to not show register page
                            PreferencesService.setApiKey($scope.data.apiKey);
                            PreferencesService.setUuid(Globals.UUID_DEMO_ACCOUNT);
                            PreferencesService.setAccountMode(1);
                            PreferencesService.setAccountAccess(data.access);
                            PreferencesService.setPushNotificationsEnable(true);
                            if (data.access) {
                                //This kind of account needs user registration
                                $state.go('register');
                            } else {
                                //This kind of account don't need client register
                                console.log("Saving appid and going directly to search contents page");
                                var appID = "da" + $cordovaDevice.getUUID();
                                PreferencesService.setAppId(appID);
                                $state.go('app.searchContents');
                            }
                        } else {
                            console.log("Other error on cheking api key: " + JSON.stringify(data));
                        }
                    }).error(function (error) {
                        showToastMessage(translations.WELCOME_INCORRECT_API_CHECK);
                    });
                } else {
                    showToastMessage(translations.WELCOME_INCORRECT_APIKEY_FORMAT);
                }
            });
        }

        // Evento al pulsar el botón de crear cuenta
        $scope.createAccount = function () {
            window.open('https://www.easycontext.com/proximity-marketing-platform', '_system', 'location=yes');
            //PreferencesService.setUuid(Globals.UUID_DEMO_ACCOUNT);
            //PreferencesService.setAccountMode(2);
            //$state.go('register');
        };

        // Evento al acceder a la app mediante la cuenta de demo
        $scope.enterDemoAccount = function () {
            // Se guarda el API-KEY y el UUID de demo en las preferencias
            var appID = "da" + $cordovaDevice.getUUID();
            PreferencesService.setAppId(appID);
            PreferencesService.setApiKey(Globals.API_KEY_DEMO_ACCOUNT);
            PreferencesService.setUuid(Globals.UUID_DEMO_ACCOUNT);
            PreferencesService.setAccountMode(0);
            PreferencesService.setPushNotificationsEnable(true);
            PreferencesService.setAccountAccess("true");
            $state.go('app.searchContents');
        };

        // Funcion para validar el API KEY
        function isValidApiKey(apikey) {
            return apikey.length == 19 && apikey.match("[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}");
        }

        // Función para mostrar mensaje de error
        function showToastMessage(text) {
            $cordovaToast.showShortBottom(text).then(function (success) {}, function (error) {});
        }
    }]);