/* global $ */
angular.module('starter.services').factory('PlatformService', ['$q', '$http', '$timeout', 'Globals', '$ionicLoading',
    function ($q, $http, $timeout, Globals, $ionicLoading) {
        return {
            // Funcion para crear o actualizar un usuario en la plataforma
            updateUserData: function (appid, apiKey, birthDate, gender, username, lables) {
                var deferred = $q.defer();
                var promise = deferred.promise;

                // Se crea la URL para mandar el GET 
                var url = Globals.URL_CREATE_UPDATE_USER;
                url = url.replace("<api_key>", apiKey);
                url = url.replace("<user_appid>", appid);
                url = url.replace("<birth_date>", birthDate);
                url = url.replace("<gender>", gender);
                url = url.replace("<feature1>", lables);
                url = url.replace("<feature2>", username);
                url = url.replace("<feature3>", "");

                console.log("URL: " + url);

                $http.get(url).success(
                        function (data) {
                            console.log(data);
                            deferred.resolve(data);
                        }
                ).error(
                        function (data) {
                            console.log(data);
                            deferred.reject(data);
                        }
                );

                promise.success = function (fn) {
                    promise.then(fn);
                    return promise;
                }
                promise.error = function (fn) {
                    promise.then(null, fn);
                    return promise;
                }
                return promise;
            },
            // Funcion para obtener los datos del usuario
            getUserData: function (appid, apiKey) {
                var deferred = $q.defer();
                var promise = deferred.promise;

                // Se crea la URL para mandar el GET 
                var url = Globals.URL_GET_USER_DATA;
                url = url.replace("<api_key>", apiKey);
                url = url.replace("<user_appid>", appid);

                console.log("URL: " + url);

                $http.get(url).success(
                        function (data) {
                            console.log(data);
                            deferred.resolve(data);
                        }
                ).error(
                        function (data) {
                            console.log(data);
                            deferred.reject(data);
                        }
                );

                promise.success = function (fn) {
                    promise.then(fn);
                    return promise;
                }
                promise.error = function (fn) {
                    promise.then(null, fn);
                    return promise;
                }
                return promise;
            },
            // Funcion para registrar un dispositivo con notificaciones push
            registerDevicePushNotification: function (appid, apiKey, platform, token) {
                var deferred = $q.defer();
                var promise = deferred.promise;

                // Se crea la URL para mandar el GET 
                var url = Globals.URL_PUSH_REGISTER;
                url = url.replace("<api_key>", apiKey);
                url = url.replace("<user_appid>", appid);
                url = url.replace("<platform>", platform);
                url = url.replace("<token>", token);

                console.log("URL: " + url);

                $http.get(url).success(
                        function (data) {
                            console.log(data);
                            deferred.resolve(data);
                        }
                ).error(
                        function (data) {
                            console.log(data);
                            deferred.reject(data);
                        }
                );

                promise.success = function (fn) {
                    promise.then(fn);
                    return promise;
                }
                promise.error = function (fn) {
                    promise.then(null, fn);
                    return promise;
                }
                return promise;
            },
            // Function to register when the user interested and opened push notification
            registerPushNotificationOpened: function (apiKey, notificationId) {
                var deferred = $q.defer();
                var promise = deferred.promise;

                // Se crea la URL para mandar el GET 
                var url = Globals.URL_PUSH_REGISTER_ACTION;
                url = url.replace("<api_key>", apiKey);
                url = url.replace("<notif_id>", notificationId);

                console.log("RegisterPushNotificationOpened URL: " + url);

                $http.get(url).success(
                        function (data) {
                            console.log(data);
                            deferred.resolve(data);
                        }
                ).error(
                        function (data) {
                            console.log(data);
                            deferred.reject(data);
                        }
                );

                promise.success = function (fn) {
                    promise.then(fn);
                    return promise;
                };

                promise.error = function (fn) {
                    promise.then(null, fn);
                    return promise;
                };

                return promise;
            },
            // Function for getting contents using ibeacon data and geolocation
            getZoneContentsIbeaconLocation: function (appid, apiKey, uuid, major, minor, latitude, longitude, action) {
                var deferred = $q.defer();
                var promise = deferred.promise;

                // Se crea la URL para mandar el GET 
                var url = Globals.URL_GET_CONTENTS_IBEACON_LOCATION;
                url = url.replace("<api_key>", apiKey);
                url = url.replace("<appid>", appid);
                url = url.replace("<uuid>", uuid);
                url = url.replace("<major>", major);
                url = url.replace("<minor>", minor);
                url = url.replace("<latitude>", latitude);
                url = url.replace("<longitude>", longitude);
                url = url.replace("<action>", action);

                console.log("7 - URL: " + url);

                $http.get(url).success(function (data) {
                    console.log("8 - Success on getZoneContentsIbeaconLocation: " + data);
                    deferred.resolve(data);
                }).error(function (error) {
                    console.log("8 - Error on getZoneContentsIbeaconLocation: " + error);
                    deferred.reject(error);
                });

                promise.success = function (fn) {
                    promise.then(fn);
                    return promise;
                }
                promise.error = function (fn) {
                    promise.then(null, fn);
                    return promise;
                }
                return promise;
            },
            getContentsByUrl: function (url) {
                var deferred = $q.defer();
                var promise = deferred.promise;

                $http.get(url).success(function (data) {
                    deferred.resolve(data);

                }).error(function (error) {
                    deferred.reject(error);
                });

                promise.success = function (fn) {
                    promise.then(fn);
                    return promise;
                }

                promise.error = function (fn) {
                    promise.then(null, fn);
                    return promise;
                }
                return promise;
            },
            // Funcion registrar la actividad del usuario
            // Type: details: the user clicked in the content to access more details and a longer description
            // Type: fav: the user added the content to favorites
            // Type: social: the user shared the content through another app
            logUserInteraction: function (appid, apiKey, type, contentId, social) {
                var deferred = $q.defer();
                var promise = deferred.promise;

                // Se crea la URL para mandar el GET 
                var url = Globals.URL_LOG_USER_INTERACTION;
                url = url.replace("<type>", type);
                url = url.replace("<api_key>", apiKey);
                url = url.replace("<appid>", appid);
                url = url.replace("<id_content>", contentId);
                url = url.replace("<social>", social);

                console.log("URL: " + url);

                $http.get(url).success(
                        function (data) {
                            console.log(data);
                            deferred.resolve(data);
                        }
                ).error(
                        function (data) {
                            console.log(data);
                            deferred.reject(data);
                        }
                );

                promise.success = function (fn) {
                    promise.then(fn);
                    return promise;
                }
                promise.error = function (fn) {
                    promise.then(null, fn);
                    return promise;
                }
                return promise;
            },
            // Funcion para comprobar si el API-KEY esta registrado
            checkApiKey: function (apiKey) {
                var deferred = $q.defer();
                var promise = deferred.promise;
                var data = JSON.stringify({apikey: apiKey, key: Globals.UNIQUE_KEY_PLATFORM}, null, 2);

                $ionicLoading.show({
                    template: '<ion-spinner icon="android"></ion-spinner>'
                });

                $.ajax({
                    url: Globals.URL_CHECK_API_KEY,
                    type: "POST",
                    data: data,
                    dataType: "json",
                    timeout: 10000,
                    success: function (response) {
                        $ionicLoading.hide();
                        console.log(JSON.stringify(response));
                        deferred.resolve(response);
                    },
                    error: function (xhr, ajaxOptions, thrownError) {
                        $ionicLoading.hide();
                        console.log("ERROR: " + JSON.stringify(xhr));
                        if (ajaxOptions === "timeout") {
                            deferred.reject("Connection timeout");
                        } else {
                            deferred.reject("Error");
                        }
                    }
                });

                promise.success = function (fn) {
                    promise.then(fn);
                    return promise;
                }
                promise.error = function (fn) {
                    promise.then(null, fn);
                    return promise;
                }
                return promise;
            },
            // Funcion para registrar un client en la plataforma
            registerClient: function (name, email) {
                var deferred = $q.defer();
                var promise = deferred.promise;
                var data = JSON.stringify({client: {name: name, mail: email}, key: Globals.UNIQUE_KEY_PLATFORM}, null, 2);

                $ionicLoading.show({
                    template: '<ion-spinner icon="android"></ion-spinner>'
                });

                $.ajax({
                    url: Globals.URL_REGISTER_CLIENT,
                    type: "POST",
                    data: data,
                    dataType: "json",
                    timeout: 10000,
                    success: function (response) {
                        $ionicLoading.hide();
                        console.log(response);
                        deferred.resolve(response);
                    },
                    error: function (xhr, ajaxOptions, thrownError) {
                        $ionicLoading.hide();
                        console.log("ERROR: " + xhr);
                        if (ajaxOptions === "timeout") {
                            deferred.reject("Connection timeout");
                        } else {
                            deferred.reject("Error");
                        }
                    }
                });

                promise.success = function (fn) {
                    promise.then(fn);
                    return promise;
                }
                promise.error = function (fn) {
                    promise.then(null, fn);
                    return promise;
                }
                return promise;
            }
        };
    }]);