/* global angular */
angular.module('starter.services').factory('PreferencesService', ["Utils",
    function (Utils) {

        // USER INFO
        var APP_ID_TAG = "app_id";
        var USERNAME_TAG = "username";
        var BIRTHDATE_TAG = "birthdate";
        var GENDER_TAG = "gender";
        var REGISTER_DATE_TAG = "register_date";

        // APP ACCESS INFO
        var ACCESS_PROVIDER_TAG = "access_provider";
        var ACCESS_TOKEN_TAG = "access_provider_token";
        var PUSH_TOKEN_TAG = "push_token";

        // USER SETTINGS
        var PUSH_NOTIFICATIONS_TAG = "push_notifications";
        var BACKGROUND_MODE_TAG = "background_mode";
        var BACKGROUND_BATTERY_SAVER_MODE = "background_battery_saver_mode";
        var GEOLOCATION_TAG = "geolocation";
        var AUTOMATIC_DISCOVERING_TAG = "auto_discover";
        var AUTOMATIC_DISCOVERING_TIMER_TAG = "auto_discover_timer";
        var BEACONS_SIMULATION_TAG = "beacons_simulation";

        // APP DEMO INFO
        var APP_ACTIVATED_TAG = "app_activated";
        var APP_ACCOUNT_MODE = "account_mode";      // 0 - Demo account, 1 - User account, 2 - User new account
        var APP_ACCOUNT_ACCESS = "account_access";      // false - is Pro or Basic account type and true - if it is advanced
        var API_KEY_TAG = "api_key";
        var UUID_TAG = "uuid";

        var FAVORITES_TAG = "favorites";

        var LABELS_TAG = "labels";

        return {
            logout: function () {
                window.localStorage.clear();
            },
            setAndr6LocAsked: function (status) {
                window.localStorage.setItem("ANDR6_LOCATION_ASKED", status);
            },
            getAndr6LocAsked: function () {
                return window.localStorage.getItem("ANDR6_LOCATION_ASKED");
            },
            setAppId: function (appId) {
                window.localStorage.setItem(APP_ID_TAG, appId);
            },
            getAppId: function () {
                return window.localStorage.getItem(APP_ID_TAG);
            },
            setUsername: function (username) {
                window.localStorage.setItem(USERNAME_TAG, username);
            },
            getUsername: function () {
                return window.localStorage.getItem(USERNAME_TAG);
            },
            setBirthdate: function (birthdate) {
                window.localStorage.setItem(BIRTHDATE_TAG, birthdate);
            },
            getBirthdate: function () {
                return window.localStorage.getItem(BIRTHDATE_TAG);
            },
            setGender: function (gender) {
                window.localStorage.setItem(GENDER_TAG, gender);
            },
            getGender: function () {
                return window.localStorage.getItem(GENDER_TAG);
            },
            setRegisterDate: function (registerDate) {
                window.localStorage.setItem(REGISTER_DATE_TAG, registerDate);
            },
            getRegisterDate: function () {
                return window.localStorage.getItem(REGISTER_DATE_TAG);
            },
            setAccessProvider: function (accessProvider) {
                window.localStorage.setItem(ACCESS_PROVIDER_TAG, accessProvider);
            },
            getAccessProvider: function () {
                return window.localStorage.getItem(ACCESS_PROVIDER_TAG);
            },
            setPushToken: function (accessToken) {
                window.localStorage.setItem(PUSH_TOKEN_TAG, accessToken);
            },
            getPushToken: function () {
                return window.localStorage.getItem(PUSH_TOKEN_TAG);
            },
            setLoginData: function (appId, username, birthday, gender, loginProvider, accessToken) {
                console.log("PreferencesService: AppID: " + appId + ", Username: " + username + ", Birthday: " + birthday + ", Gender: " + gender + ", LoginProvider: " + loginProvider + ", AccessToken: " + accessToken);
                window.localStorage.setItem(APP_ID_TAG, appId);
                window.localStorage.setItem(USERNAME_TAG, username);
                window.localStorage.setItem(BIRTHDATE_TAG, birthday);
                window.localStorage.setItem(GENDER_TAG, gender);
                window.localStorage.setItem(ACCESS_PROVIDER_TAG, loginProvider);
                window.localStorage.setItem(ACCESS_TOKEN_TAG, accessToken);
                window.localStorage.setItem(REGISTER_DATE_TAG, Utils.formatDate(new Date()));
            },
            updateProfileData: function (birthdate, gender, labels) {
                window.localStorage.setItem(BIRTHDATE_TAG, birthdate);
                window.localStorage.setItem(GENDER_TAG, gender);
                window.localStorage.setItem(LABELS_TAG, labels);
            },
            getLoginData: function () {
                return {
                    BIRTHDATE_TAG: window.localStorage.getItem(BIRTHDATE_TAG),
                    APP_ID_TAG: window.localStorage.getItem(APP_ID_TAG),
                    USERNAME_TAG: window.localStorage.getItem(USERNAME_TAG),
                    GENDER_TAG: window.localStorage.getItem(GENDER_TAG),
                    LABELS_TAG: (window.localStorage.getItem(LABELS_TAG) !== null) ? window.localStorage.getItem(LABELS_TAG).split(",") : null,
                    REGISTER_DATE_TAG: window.localStorage.getItem(REGISTER_DATE_TAG)
                };
            },
            setPushNotificationsEnable: function (enable) {
                window.localStorage.setItem(PUSH_NOTIFICATIONS_TAG, enable);
            },
            getPushNotificationsEnable: function () {
                return window.localStorage.getItem(PUSH_NOTIFICATIONS_TAG) == "true";
            },
            setGeolocation: function (enable) {
                window.localStorage.setItem(GEOLOCATION_TAG, enable);
            },
            getGeolocation: function () {
                return window.localStorage.getItem(GEOLOCATION_TAG) == "true";
            },
            setAutoDiscovering: function (enable) {
                window.localStorage.setItem(AUTOMATIC_DISCOVERING_TAG, enable);
            },
            getAutoDiscovering: function () {
                return window.localStorage.getItem(AUTOMATIC_DISCOVERING_TAG) == "true";
            },
            setAutoDiscoveringTimer: function (value) {
                window.localStorage.setItem(AUTOMATIC_DISCOVERING_TIMER_TAG, value);
            },
            getAutoDiscoveringTimer: function () {
                return window.localStorage.getItem(AUTOMATIC_DISCOVERING_TIMER_TAG);
            },
            setBeaconsSimulation: function (enable) {
                window.localStorage.setItem(BEACONS_SIMULATION_TAG, enable);
            },
            getBeaconsSimulation: function () {
                return window.localStorage.getItem(BEACONS_SIMULATION_TAG) == "true";
            },
            setAccountMode: function (accountMode) {
                window.localStorage.setItem(APP_ACCOUNT_MODE, accountMode);
            },
            getAccountMode: function () {
                return window.localStorage.getItem(APP_ACCOUNT_MODE);
            },
            setAccountAccess: function (account_access) {
                window.localStorage.setItem(APP_ACCOUNT_ACCESS, account_access);
            },
            getAccountAccess: function () {
                return window.localStorage.getItem(APP_ACCOUNT_ACCESS);
            },
            setBackgroundMode: function (enable) {
                window.localStorage.setItem(BACKGROUND_MODE_TAG, enable);
            },
            getBackgroundMode: function () {
                return window.localStorage.getItem(BACKGROUND_MODE_TAG) == "true";
            },
            setBackgroundBatterySaverMode: function (enable) {
                window.localStorage.setItem(BACKGROUND_BATTERY_SAVER_MODE, enable);
            },
            getBackgroundBatterySaverMode: function () {
                return window.localStorage.getItem(BACKGROUND_BATTERY_SAVER_MODE) == "true";
            },
            setApiKey: function (apiKey) {
                window.localStorage.setItem(API_KEY_TAG, apiKey);
            },
            getApiKey: function () {
                return window.localStorage.getItem(API_KEY_TAG);
            },
            setUuid: function (uuid) {
                window.localStorage.setItem(UUID_TAG, uuid);
            },
            getUuid: function () {
                return window.localStorage.getItem(UUID_TAG);
            },
            setAppActivated: function (activated) {
                window.localStorage.setItem(APP_ACTIVATED_TAG, activated);
            },
            isAppActivated: function () {
                return window.localStorage.getItem(APP_ACTIVATED_TAG) == "true";
            },
            addFavoriteContent: function (content) {
                var contents = angular.fromJson(window.localStorage[FAVORITES_TAG]) || [];
                content.date = new Date();
                contents.unshift(content);
                window.localStorage[FAVORITES_TAG] = angular.toJson(contents);
            },
            favoriteContentExists: function (contentId) {
                var contents = angular.fromJson(window.localStorage[FAVORITES_TAG]) || [];
                var contentExists = false;
                angular.forEach(contents, function (content) {
                    if (content.id === contentId) {
                        contentExists = true;
                    }
                });
                return contentExists;
            },
            getFavoriteContents: function () {
                return angular.fromJson(window.localStorage[FAVORITES_TAG]) || [];
            },
            removeFavoriteContent: function (contentId) {
                var contents = angular.fromJson(window.localStorage[FAVORITES_TAG]) || [];
                var index = -1;
                contents.find(function (item, i) {
                    if (item.id === contentId) {
                        index = i;
                    }
                });
                if (index !== -1) {
                    contents.splice(index, 1);
                    window.localStorage[FAVORITES_TAG] = angular.toJson(contents);
                }
            },
        };
    }]);