/* global angular */
angular.module('starter.services').factory('PushNotificationsService', ['$cordovaPushV5', '$ionicHistory', 'Globals', 'PlatformService', 'PreferencesService', '$rootScope', '$translate', '$state', '$ionicPopup',
    function ($cordovaPushV5, $ionicHistory, Globals, PlatformService, PreferencesService, $rootScope, $translate, $state, $ionicPopup) {

        var registered = false;

        var options = {
            android: {
                senderID: Globals.INFO().PUSH_NOTIFICATION_SENDER_ID
            },
            ios: {
                alert: "true",
                badge: "true",
                sound: "true"
            }
        };

        $rootScope.$on('$cordovaPushV5:notificationReceived', function (event, data) {
            console.log("PUSH Received: " + JSON.stringify(data));

            // Server sent Push Notification
            if (data.additionalData != undefined && data.additionalData.url != undefined && data.additionalData.type == 'campaign') {

                console.log("Date: " + new Date().getTime() / 1000);
                console.log("Date2: " + Globals.START_TIMESTAMP);

                // Check if app was opened before receiving the notification
                if ((new Date().getTime() / 1000) - Globals.START_TIMESTAMP < 20) {
                    // Register the opened acion to server
                    PlatformService.registerPushNotificationOpened(PreferencesService.getApiKey(), data.additionalData.notification_id);

                    PlatformService.getContentsByUrl(data.additionalData.url).success(function (data) {
                        Globals.LAST_DISCOVERED_CONTENTS = JSON.stringify(data);
                        console.log(JSON.stringify($state.current));
                        $state.go('app.searchContents', {}, {reload: true});

                    }).error(function (error) {
                        console.log("Error: " + error);
                    });
                } else {
                    $translate(['BUTTON_CANCEL', 'BUTTON_ACCEPT']).then(function (translations) {
                        $ionicPopup.confirm({
                            title: data.title,
                            template: '<p align="center">' + data.message + '</p>',
                            buttons: [
                                {
                                    text: translations.BUTTON_CANCEL,
                                    type: 'button-default',
                                    onTap: function (e) {}
                                },
                                {
                                    text: translations.BUTTON_ACCEPT,
                                    type: 'button-positive',
                                    onTap: function (e) {
                                        // Register the opened acion to server
                                        PlatformService.registerPushNotificationOpened(PreferencesService.getApiKey(), data.additionalData.notification_id);

                                        PlatformService.getContentsByUrl(data.additionalData.url).success(function (data) {
                                            Globals.LAST_DISCOVERED_CONTENTS = JSON.stringify(data);
                                            console.log(JSON.stringify($state.current));
                                            $state.go('app.searchContents', {}, {reload: true});

                                        }).error(function (error) {
                                            console.log("Error: " + error);
                                        });
                                    }
                                }
                            ]
                        });
                    });
                }
            } else if (data.additionalData != undefined && data.additionalData.type != undefined && data.additionalData.type == 'get') {
                if (cordova.plugins.backgroundMode.isActive() || ((new Date().getTime() / 1000) - Globals.START_TIMESTAMP < 4)) {
                    Globals.START_SEARCHING_CONTENTS = true;
                    console.log("Push Get will open searchcontent");
                    $state.go('app.searchContents', {}, {reload: true});
                } else {
                    console.log("Push get recieved but app is active, so it will not be reloaded.");
                }

            }

        });

        return {
            // Function to register push notifications in platform
            registerPushNotifications: function () {
                if (PreferencesService.getPushNotificationsEnable() && !registered) {
                    $cordovaPushV5.initialize(options).then(function () {
                        // start listening for new notifications
                        $cordovaPushV5.onNotification();

                        // register to get registrationId
                        $cordovaPushV5.register().then(function (id) {
                            console.log("PUSH: " + id);

                            // Saving in preferences the PUSH TOKEN
                            PreferencesService.setPushToken(id);

                            var platform = (cordova.platformId == 'android') ? 'gcm' : 'apns';

                            console.log("Account access: " + PreferencesService.getAccountAccess());
                            if (PreferencesService.getAccountAccess() == "true") {
                                PlatformService.registerDevicePushNotification(PreferencesService.getAppId(), PreferencesService.getApiKey(), platform, id).success(function (data) {
                                    registered = true;
                                }).error(function (error) {
                                    console.log(error);
                                    registered = false;
                                });
                            } else {
                                // If account don't have access like if it type of Pro or Basic
                                registered = true;
                                console.log("Device will not be registered to platform push notification");
                            }
                        });
                    });
                } else {
                    console.log("Push will not be registered: Settings:" + PreferencesService.getPushNotificationsEnable() + " Registered:" + registered);
                }
            },
            // Function to unregister push notifications
            unregisterPushNotifications: function () {
                if (registered) {
                    $cordovaPushV5.unregister().then(function (result) {
                        console.log(JSON.stringify(result));
                        PreferencesService.setPushToken(null);
                        registered = false;
                    });
                } else {
                    console.log("Push will not be unregistered: Registered:" + registered);
                }
            }
        };
    }]);